// HEADER - BlitBitmap.h
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, September 2012
//
// Bitmap blitting.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#if !defined(__BLITBITMAP_H__)
#define __BLITBITMAP_H__

#define SCREEN_WIDTH_IN_PIXELS 320
#define SCREEN_HEIGHT_IN_PIXELS 256
#define SCREEN_HEIGHT_VISIBLE_IN_PIXELS 212
#define SCREEN_CROP_HEIGHT_IN_PIXELS (SCREEN_HEIGHT_IN_PIXELS-SCREEN_HEIGHT_VISIBLE_IN_PIXELS)
#define SCREEN_WIDTH_IN_BYTES (SCREEN_WIDTH_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)

#define GRAPHICS_COLOUR_DEPTH 16        // 16BPP
#define PIXEL_WIDTH_IN_BYTES (GRAPHICS_COLOUR_DEPTH/8)

typedef struct
    {
    unsigned char* iData;       // Bitmap data.
    unsigned char* iDest;       // Destination.
    unsigned long int iFlags2;  // Blitter unit A2 flags (additional to normal).
    unsigned long int iCmd;     // Blitter commands.
    unsigned short iWidth;      // Bitmap size.
    unsigned short iHeight;     // Bitmap size.
    unsigned short iX;          // Where its going.
    unsigned short iY;          // Where its going.
    } TBitmapBlitType;

extern unsigned char *gameBackScreen;
extern unsigned char *gameFrontScreen;

extern void BlitClearScreen(void);
extern void BlitBitmap(const TBitmapBlitType* aBmp);
extern void BlitFrontToBack(void);
extern unsigned short BlitMemCpy64(void* aDestPtr,void* aSrcPtr,unsigned long aSizeInBytes);

// Jaguar h/w
// Blitter.
#define WID10   0x00001A00
#define WID16   0x00002000
#define WID320  0x00004200
#define LFU_REPLACE (0xC<<21)
#define DCOMPEN     (1<<27)

// 256 colour palette.
#define CLUT 0xF00400

#endif // __BLITBITMAP_H__
