; MODULE - BlitBitmap.s
;
; Copyright (c) 2012 Mark Ball. All rights reserved.
; Started by Mark Ball, August 2012
;
; Bitmap to display buffer blitter.
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;

; ------------------------------------------------------------------
; Includes.
; ------------------------------------------------------------------
    .include "jaguar.inc"
    .include "game.inc"

; ------------------------------------------------------------------
; Imports.
; ------------------------------------------------------------------
    .extern _gameBackScreen
    .extern _gameFrontScreen

; ------------------------------------------------------------------
; Equates.
; ------------------------------------------------------------------
; typedef struct
;    {
;    unsigned char* iData;       // Bitmap data.
;    unsigned char* iDest;       // Destination.
;    unsigned long int iFlags2;  // Blitter unit A2 flags (additional to normal).
;    unsigned long int iCmd;     // Blitter commands.
;    unsigned short iWidth;      // Bitmap size.
;    unsigned short iHeight;     // Bitmap size.
;    unsigned short iX;          // Where its going.
;    unsigned short iY;          // Where its going.
;    } TBitmapBlitType;

    .cargs #0, iData.l, iDest.l, iFlagsA2.l, iCmd.l, iSize_iWidth.w, iSize_iHeight.w, iDest_iX.w, iDest_iY.w

; ------------------------------------------------------------------
; Code starts here.
; ------------------------------------------------------------------
    .text
    .68000

; ------------------------------------------------------------------
; _BlitClearScreen
; ------------------------------------------------------------------
; Blit the back buffer clear.
; ------------------------------------------------------------------
; extern void BlitClearScreen(void);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _BlitClearScreen
_BlitClearScreen:

.WaitForBlitter:
    move.l B_CMD, d0
    btst #0, d0
    beq.s .WaitForBlitter

; Source - black.
    move.l #0, B_PATD
    move.l #0, B_PATD+4

; Destination.
    move.l #0, A1_CLIP
    move.l _gameBackScreen, A1_BASE
    move.l #PITCH1|PIXEL32|WID160|XADDPHR, A1_FLAGS
    move.l #0, A1_PIXEL
    move.l #(1<<16)|(-(SCREEN_WIDTH_IN_PIXELS/2)&$FFFF), A1_STEP
    move.l #(SCREEN_HEIGHT_IN_PIXELS<<16)|(SCREEN_WIDTH_IN_PIXELS/2), B_COUNT

; Start it going.
    move.l #PATDSEL|UPDA1|LFU_REPLACE, B_CMD
    rts

; ------------------------------------------------------------------
; _BlitFrontToBack
; ------------------------------------------------------------------
; Blit the front buffer to the back buffer.
; ------------------------------------------------------------------
; extern void BlitFrontToBack(void);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _BlitFrontToBack
_BlitFrontToBack:

.WaitForBlitter:
    move.l B_CMD, d0
    btst #0, d0
    beq.s .WaitForBlitter

; Source.
    move.l _gameFrontScreen, A2_BASE
    move.l #0, A2_PIXEL
    move.l #PITCH1|PIXEL32|WID160|XADDPHR, A2_FLAGS
    move.l #(0<<16)|0, A2_STEP

; Destination.
    move.l #0, A1_CLIP
    move.l _gameBackScreen, A1_BASE
    move.l #PITCH1|PIXEL32|WID160|XADDPHR, A1_FLAGS
    move.l #0, A1_PIXEL
    move.l #(1<<16)|(-(SCREEN_WIDTH_IN_PIXELS/2)&$FFFF), A1_STEP
    move.l #(SCREEN_HEIGHT_IN_PIXELS<<16)|(SCREEN_WIDTH_IN_PIXELS/2), B_COUNT

; Start it going.
    move.l #SRCEN|UPDA1|UPDA2|LFU_REPLACE, B_CMD
    rts

; ------------------------------------------------------------------
; _BlitBitmap
; ------------------------------------------------------------------
; Blit a bitmap from one to another.
; ------------------------------------------------------------------
; extern void BlitBitmap(const TBitmapBlitType* aBmp);
; ------------------------------------------------------------------
;
; In :-
;   4(sp) - Pointer to a TBitmapBlitType structure.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _BlitBitmap

    .cargs #4, aBlitBmpPtr.l

_BlitBitmap:

.WaitForBlitter:
    move.l B_CMD, d0
    btst #0, d0
    beq.s .WaitForBlitter

; Address unit A1 is destination, and A2 the source.
    move.l #(SCREEN_HEIGHT_IN_PIXELS<<16)|SCREEN_WIDTH_IN_PIXELS, A1_CLIP
    move.l #0, B_PATD
    move.l #0, B_PATD+4

    move.l aBlitBmpPtr(sp), a0
    move.l iData(a0), d0
    move.l d0, d1
    and.b #$F8, d1                      ; Ensure phrase aligned.
    move.l d1, A2_BASE                  ; A2_BASE=pSrc&0xFFFFFFF8;
    and.l #$00000007, d0
    move.l d0, A2_PIXEL                 ; A2_PIXEL=pSrc&0x7;

    move.l iFlagsA2(a0), d0             ; Get A2 flags.
    or.l #PITCH1|PIXEL16|XADDPIX, d0     ; Or in the defaults.
    move.l d0, A2_FLAGS  ;
    move.l #(0<<16)|0, A2_STEP          ; step = 0,0

; Setup destination.
    move.l iDest(a0), A1_BASE
    move.l #PITCH1|PIXEL16|WID320|XADDPIX, A1_FLAGS        ; No phrase gap, 16 BPP, 
    move.l iDest_iX(a0), d0
    swap.w d0
    move.l d0, A1_PIXEL
    move.l #(1<<16), d0
    move.w iSize_iWidth(a0), d0
    neg.w d0
    move.l d0, A1_STEP
    move.l iSize_iWidth(a0), d0
    swap.w d0
    move.l d0, B_COUNT

; Start it going.
    move.l iCmd(a0), d0
    or.l #SRCEN|UPDA1|UPDA2|CLIP_A1, d0
    move.l d0, B_CMD
    rts

; ------------------------------------------------------------------
; _BlitMemCpy64
; ------------------------------------------------------------------
; Blit a phrase aligned source and destination bitmap.
; ------------------------------------------------------------------
; extern unsigned short BlitMemCpy64(void* aDestPtr,void* aSrcPtr,unsigned long aSizeInBytes);
; ------------------------------------------------------------------
;
; In :-
;   4(sp) - Pointer to a destination.
;   8(sp) - Pointer to a source.
;   12(sp) - Length in bytes to copy.
;
; Out :-
;   d0 - 0 if OK, +ve otherwise.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
;
; Notes:-
; 1) Source and destination must not overlap.
;
; ------------------------------------------------------------------
    .globl _BlitMemCpy64

    .cargs #4, aDestMemCpyPtr.l, aSrcMemCpyPtr.l, aSizeMemCpyInBytes.l

_BlitMemCpy64:
    move.l aDestMemCpyPtr(sp), a0       ; Destination bitmap pointer.
    move.l a0, d0
    and.b #$7, d0                       ; Phrase aligned?
    bne.s .DestinationNotPhraseAligned  ; No! Can't handle it.

    move.l aSrcMemCpyPtr(sp), a1        ; Source bitmap pointer.
    move.l a1, d0
    and.b #$7, d0                       ; Phrase aligned?
    bne.s .SourceNotPhraseAligned       ; No! Can't handle it.

    move.l aSizeMemCpyInBytes(sp), d1   ; Size to copy.
    cmp.l #$4000, d1                    ; > 0x4000?
    bhi.s .TooManyBytesToCopy           ; Yes... To much!

; Wait for the blitter.
.WaitForBlitter:
    move.l B_CMD, d0
    btst #0, d0
    beq.s .WaitForBlitter

    clr.l d0

; Setup the source.
    move.l a1, A2_BASE
    move.l d0, A2_PIXEL
    move.l #PIXEL8|WID8|XADDPHR, A2_FLAGS

; Setup the destination.
    move.l a0, A1_BASE
    move.l d0, A1_CLIP
    move.l d0, A1_PIXEL
    move.l d0, A1_FPIXEL
    move.l #PIXEL8|WID8|XADDPHR, A1_FLAGS

; Count.
    bset #16, d1
    move.l d1, B_COUNT

; Start the blitter.
    move.l #SRCEN|LFU_REPLACE, B_CMD
    rts

; An error occurred.
.TooManyBytesToCopy:
.SourceNotPhraseAligned:
.DestinationNotPhraseAligned:
    moveq.l #1, d0
    rts

; ------------------------------------------------------------------
; END
; ------------------------------------------------------------------
    .end

