//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CHUCKIE_H
#define CHUCKIE_H

// From stdint.h
typedef unsigned char uint8_t;
typedef unsigned long int uint32_t;

/*** Game interface.  */

#define BUTTON_RIGHT  1
#define BUTTON_LEFT   2
#define BUTTON_DOWN   4
#define BUTTON_UP     8
#define BUTTON_JUMP   0x10
#define BUTTON_QUIT   0x80
extern uint8_t buttons;
extern uint8_t button_ack;
extern int cheat;
extern int skip_frame;

void PollKeys(void);
void RenderFrame(void);
void RenderBackground(void);

void run_game(void);

/*** Audio interface */

#define BUFSIZE 512
#define FREQ 44100
extern short next_buffer[BUFSIZE];
void mix_buffer(void);
void sound_start(int, int);
void init_sound(void);
void init_input(void);

/*** Misc.  */
extern unsigned char* gameLevels[8];

/*** Data for rendering engine.  */

#define TILE_WALL     1
#define TILE_LADDER   2
#define TILE_EGG      4
#define TILE_GRAIN    8

/* Direction bitflags  */
#define DIR_L		1
#define DIR_R		2
#define DIR_UP		4
#define DIR_DOWN	8
#define DIR_HORIZ	(DIR_R | DIR_L)

typedef enum {
    PLAYER_WALK,
    PLAYER_CLIMB,
    PLAYER_JUMP,
    PLAYER_FALL,
    PLAYER_LIFT
} player_mode_t;

extern int current_player;
extern int num_players;
extern int current_level;

extern int player_x;
extern int player_y;
extern player_mode_t player_mode;
extern int player_face;
extern int move_x;
extern int move_y;
extern uint8_t bonus[4];
extern uint8_t timer_ticks[3];
extern uint8_t levelmap[20 * 25];
extern int have_lift;
extern int lift_x;
extern int lift_y[2];

extern int have_big_duck;
extern int big_duck_frame;
extern int big_duck_x;
extern int big_duck_y;
extern int big_duck_dir;

typedef enum {
    DUCK_BORED,
    DUCK_STEP,
    DUCK_EAT1,
    DUCK_EAT2,
    DUCK_EAT3,
    DUCK_EAT4
} duck_state;

typedef struct {
    uint8_t x;
    uint8_t y;
    uint8_t tile_x;
    uint8_t tile_y;
    duck_state mode;
    int dir;
} duckinfo_t;
extern duckinfo_t duck[5];
extern int num_ducks;

typedef struct {
    uint8_t score[8];
    uint8_t bonus[4];
    uint8_t egg[16];
    uint8_t grain[16];
    int lives;
} playerdata_t;

extern playerdata_t all_player_data[4];

int DuckSprite(int n);

extern void SoundEffectWalking(void);
extern void SoundEffectLadder(void);
extern void SoundEffectFall(void);
extern void SoundEffectJump(void);
extern void SoundEffectEgg(void);
extern void SoundEffectGrain(void);
extern void SoundEffectDeathTune(void);
extern void SoundEffectSilence(void);
extern void SoundEffectBonusStart(void);
extern void SooundEffectBonus(void);
extern void SoundEffectSilenceAll(void);

#endif
