; MODULE - EFFECTSA.S
;
; Copyright (c) 2012 Mark Ball. All rights reserved.
; Started by Mark Ball, September 2012
;
; Bitmap effects.
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;

; ------------------------------------------------------------------
; Includes
; ------------------------------------------------------------------
    .include "jaguar.inc"
    .include "game.inc"

; ------------------------------------------------------------------
; Imports.
; ------------------------------------------------------------------
    .extern _KOptionsSelectMaskBitmap
    .extern _gameBackdropBuffer

; ------------------------------------------------------------------
; Equates
; ------------------------------------------------------------------
RGB_MASK_RED    equ ($1F<<11)
RGB_MASK_GREEN  equ $3F
RGB_MASK_BLUE   equ ($1F<<6)

BAR_WIDTH_IN_PIXELS     equ 160
BAR_HEIGHT_IN_PIXELS    equ 23

; ------------------------------------------------------------------
; Code starts here.
; ------------------------------------------------------------------
    .text
    .68000

; ------------------------------------------------------------------
; _InitGpu
; ------------------------------------------------------------------
; Initialise the GPU.
; ------------------------------------------------------------------
; extern void InitGpu(void);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _InitGpu
_InitGpu:
    lea _GpuLoad, a0
    move.l #G_RAM, a1
    move.w #1024-1, d0      ; 1K x 32 bit wide.
.Loop:
    move.l (a0)+, (a1)+
    dbra d0, .Loop

; Start TOM waiting for events.
	move.l #_GpuExec, G_PC      ; Set GPU PC
	move.l #RISCGO, G_CTRL  	; Go!
    rts

; ------------------------------------------------------------------
; _AddBitmapToDimList
; ------------------------------------------------------------------
; Add a bitmap to be dimmed to the dispatch list.
; ------------------------------------------------------------------
; extern void AddBitmapToDimList(unsigned short int* aBitmapPtr,
;                    unsigned long int aLength,
;                    unsigned short int aFraction);
; ------------------------------------------------------------------
;
; In :-
;   4(sp) - Pointer to bitmap data.
;   8(sp) - Length of data to be processed in bytes.
;   12(sp) - Fraction to dim by (0 to 255).
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _AddBitmapToDimList

    .cargs #4, aBitmapPtr.l, aLengthInBytes.l, aFraction.l

_AddBitmapToDimList:
    move.l theEffect1Ptr, a0
    move.l aBitmapPtr(sp), (a0)+
    move.l aLengthInBytes(sp), (a0)+
    move.l aFraction(sp), (a0)+
    move.l a0, theEffect1Ptr
    rts

; ------------------------------------------------------------------
; _DimBitmapDispatch
; ------------------------------------------------------------------
; Process the bitmaps to be dimmed list.
; ------------------------------------------------------------------
; extern void DimBitmapDispatch(void);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _DimBitmapDispatch
_DimBitmapDispatch:
    move.l theEffect1Ptr, a0
    clr.l (a0)                              ; NULL terminate.
    move.l #1, theEventCommand              ; Dispatch it!
    move.l #theParamBuffer, theEffect1Ptr   ; Reset the pointer for next time.

; Wait for TOM to finish.
.WaitForDone:
    tst.l theEventCommand
    bne.s .WaitForDone
    rts

; ------------------------------------------------------------------
; RAM - DATA
; ------------------------------------------------------------------
    .data
    .even
theEffect1Ptr:
    dc.l theParamBuffer

    .text
    .68000

; ------------------------------------------------------------------
; _AlphaOptionsBar
; ------------------------------------------------------------------
; Pseudo alpha blend.
; ------------------------------------------------------------------
; extern void AlphaOptionsBar(const unsigned char* aDataPtr);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _AlphaOptionsBar
_AlphaOptionsBar:
    move.l aBitmapPtr(sp), theEventParam    ; Pass the pointer.
    move.l #2, theEventCommand              ; Dispatch it!

; Wait for TOM to finish.
.WaitForDone:
    tst.l theEventCommand
    bne.s .WaitForDone
    rts

; ------------------------------------------------------------------
; _DecompressBackdrop
; ------------------------------------------------------------------
; Decompress backdrops.
; ------------------------------------------------------------------
; extern void DecompressBackdrop(unsigned char* aSrc, unsigned short aType);
; ------------------------------------------------------------------
;
; In :-
;   4(sp) - Compressed data pointer.
;   8(sp) - Compression type.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _DecompressBackdrop

    .cargs #4, aCompressedData.l, aCompressType.l

_DecompressBackdrop:
    move.l aCompressedData(sp), theEventParam   ; Pass the pointer.
    move.l aCompressType(sp), theParamBuffer    ; And type.
    move.l #3, theEventCommand                  ; Dispatch it!
    rts

; ------------------------------------------------------------------
; TOM - Bitmap effects.
; ------------------------------------------------------------------
    .long
    .globl _GpuLoad
_GpuLoad:
    .gpu

    .org G_RAM
theEventCommand:
    dc.l 0
theEventParam:
    dc.l 0

    .globl _GpuExec 
_GpuExec:
    movei #theEventCommand, r31
    movei #.WaitForEventLoop, r30
    movei #theParamBuffer, r29
    movei #theEventParam, r28
    movei #.HandleEffect2, r27
    movei #.HandleEffect3, r26

; Dispatcher.
.WaitForEventLoop:
    load (r31), r0
    cmpq #0, r0
    jr EQ, .WaitForEventLoop
    cmpq #1, r0
    jr EQ, .HandleEffect1
    cmpq #2, r0
    jump EQ, (r27)
    cmpq #3, r0
    jump EQ, (r26)

    moveq #0, r0
    store r0, (r31)
    jr .WaitForEventLoop
    nop

; Effect  #1
.HandleEffect1:
    move r29, r0
    movei #.HandleEffectNextChunkLoop, r22
.HandleEffectNextChunkLoop:
    load (r0), r1
    cmpq #0, r1
    jr NE, .Effect1NotDoneYet
    addq #4, r0

.HandleEffect1Done:
    moveq #0, r0
    store r0, (r31)
    jump (r30)
    nop

.Effect1NotDoneYet:
    load (r0), r2
    addq #4, r0
    movei #.SkipDimFunction, r20
    movei #.DimLoop, r21
    load (r0), r3
    addq #4, r0
    movei #RGB_MASK_RED, r10
    movei #RGB_MASK_GREEN, r11
    movei #RGB_MASK_BLUE, r12

.DimLoop:
    loadw (r1), r4
    cmpq #0, r4
    jump EQ, (r20)

    move r4, r5
    move r4, r6

    and r10, r5
    and r11, r6

    mult r3, r5
    and r12, r4
    mult r3, r6
    shrq #8, r5
    shrq #8, r6
    mult r3, r4
    and r10, r5
    and r11, r6
    shrq #8, r4
    or r5, r6
    and r12, r4
    or r6, r4

    cmpq #0, r4
    jr NE, .SkipDimFunction
    nop
    bset #6, r4
.SkipDimFunction:
    storew r4, (r1)
    addq #2, r1
    subq #2, r2
    jump NE, (r21)
    nop
    jump (r22)
    nop

; Effect  #2
.HandleEffect2:
    load (r28),  r1
    movei #.AlphaLoopX, r20
    movei #.AlphaLoopY, r21
    movei #.SkipAlphaFunction, r22
    movei #320, r25
    movei #_KOptionsSelectMaskBitmap, r8
    movei #RGB_MASK_RED, r10
    movei #RGB_MASK_GREEN, r11
    movei #RGB_MASK_BLUE, r12

    movei #BAR_HEIGHT_IN_PIXELS, r23

.AlphaLoopY:
    movei #BAR_WIDTH_IN_PIXELS, r24

.AlphaLoopX:
    loadb (r8), r3
    addq #1, r8
    loadw (r1), r4
    cmpq #0, r4
    jump EQ, (r22)

    move r4, r5
    move r4, r6

    and r10, r5
    and r11, r6

    mult r3, r5
    and r12, r4
    mult r3, r6
    shrq #8, r5
    shrq #8, r6
    mult r3, r4
    and r10, r5
    and r11, r6
    shrq #8, r4
    or r5, r6
    and r12, r4
    or r6, r4

.SkipAlphaFunction:
    storew r4, (r1)
    addq #2, r1
    subq #1, r24
    jump NE, (r20)
    nop

    add r25, r1
    subq #1, r23
    jump NE, (r21)
    nop

    moveq #0, r0
    store r0, (r31)
    jump (r30)
    nop

.HandleEffect3:
    load (r28), r0
    movei #$C0, r1
    movei #$3F, r2
    movei #_gameBackdropBuffer, r15
    load (r29), r18
    movei #GAME_BACKDROP_WIDTH_IN_BYTES, r16
    movei #.LoopX, r17
    movei #GAME_BACKDROP_HEIGHT_IN_PIXELS, r19
    movei #.LoopY, r20
.LoopY:

    cmpq #0, r18
    move r15, r14
    jr NE, .SkipBSU
    nop
    movei #theParamBuffer, r14
.SkipBSU:

    movei #$80, r3
    moveq #0, r4
.LoopX:
    loadb (r0), r5
    move r5, r6
    and r1, r5
    addq #1, r0
    cmp r1, r5
    jr EQ, .RIsGo
    nop
    shlq #8, r3
    jr CC, .LoopX
    or r6, r3
    store r3, (r14+r4)
    addq #4, r4
    movei #$80, r3
    jr .CREol
    nop

.RIsGo:
    and r2, r6
    loadb (r0), r5
    addq #1, r0

.RUnpckLp:
    shlq #8, r3
    jr CC, .CREnd
    or r5, r3
    store r3, (r14+r4)
    addq #4, r4
    movei #$80, r3

.CREnd:
    subq #1, r6
    jr NE, .RUnpckLp 
    nop

.CREol:
    cmp r16, r4
    jump NE, (r17)
    nop

    movei #.SkipPDepack, r3
    cmpq #0, r18
    jump NE, (r3)
    nop

    movei #40, r5
    movei #80, r6
    movei #120, r7
    add r14, r5
    add r14, r6
    add r14, r7

    moveq #0, r4
    movei #40/4, r13
    movei #.PDPLoop, r3
.PDPLoop:
    load (r14), r8
    addq #4, r14
    load (r5), r9
    addq #4, r5
    load (r6), r10
    addq #4, r6
    load (r7), r11
    addq #4, r7

    .rept 4
    moveq #0, r12

    .rept 8
    shlq #1, r11
    addc r12, r12

    shlq #1, r10
    addc r12, r12

    shlq #1, r9
    addc r12, r12

    shlq #1, r8
    addc r12, r12
    .endr

    store r12, (r15+r4)
    addq #4, r4
    .endr

    subq #1, r13
    jump NE, (r3)
    nop

.SkipPDepack:
    add r16, r15
    subq #1, r19
    jump NE, (r20)
    nop

    moveq #0, r0
    store r0, (r31)
    jump (r30)
    nop         ; Don't execute buffer data as code.

; Buffer.
    .long
theParamBuffer:

; ------------------------------------------------------------------
; END
; ------------------------------------------------------------------
    .end
