// MODULE - FONT.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, September 2012
//
// Font handling.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"

extern const unsigned char KlargeFontBitmap[];
extern const unsigned char KSmallFontBitmap[];

unsigned char gameTextIsCentred=FALSE;

static TBitmapBlitType theFont=
    {
    NULL,
    NULL,
    WID16,
    DCOMPEN|LFU_REPLACE,
    LARGE_FONT_WIDTH_IN_PIXELS,LARGE_FONT_HEIGHT_IN_PIXELS,
    0,0
    };

static unsigned char* theFontPtr=(unsigned char*)&KlargeFontBitmap[0];

void SetFontSize(unsigned short int aFontSize)
//
//
//
    {

    theFont.iDest=gameBackScreen;

    if (aFontSize==FONT_USED_IS_LARGE)
        {
        theFont.iWidth=LARGE_FONT_WIDTH_IN_PIXELS;
        theFont.iHeight=LARGE_FONT_HEIGHT_IN_PIXELS;
        theFontPtr=(unsigned char*)&KlargeFontBitmap[0];
        }
    else
        {
        theFont.iWidth=SMALL_FONT_WIDTH_IN_PIXELS;
        theFont.iHeight=SMALL_FONT_HEIGHT_IN_PIXELS;
        theFontPtr=(unsigned char*)&KSmallFontBitmap[0];
        }
    }

void DrawFont(const char* aMsg,short int aX, short int aY)
//
//
//
    {
    const int len=strlen(aMsg);

    // Prep bitmap.
    theFont.iX=aX;
    theFont.iY=aY+SCREEN_CROP_HEIGHT_IN_PIXELS+6;

    if (gameTextIsCentred)
        theFont.iX-=(len*theFont.iWidth)>>1;

    // Display message.
    char *pD=(char*)aMsg;
    for (short int c=0; c<len; c++)
        {
        unsigned char skip=FALSE;
        unsigned char d=*pD++;

        if ((d>='A')&&(d<='Z'))
            d+=FONT_LETTER_INDEX-'A';
        else if ((d>='0')&&(d<='9'))
            d+=FONT_DIGIT_INDEX-'0';
        else if (d==' ')
            skip=TRUE;
        else if (d=='.')
            d=FONT_FULL_STOP_INDEX;
        else if (d==',')
            d=FONT_COMMA_INDEX;
        else if (d=='-')
            d=FONT_MINUS_INDEX;
        else if (d=='(')
            d=FONT_LEFT_BRACKET_INDEX;
        else if (d==')')
            d=FONT_RIGHT_BRACKET_INDEX;
        else if (d=='&')
            d=FONT_AMPERSAND_INDEX;
        else if (d=='!')
            d=FONT_PLING_INDEX;
        else
            d=FONT_QUESTION_MARK_INDEX;

        if (!skip)
            {
            theFont.iData=&theFontPtr[d*theFont.iWidth*theFont.iHeight*2];
            BlitBitmap(&theFont);
            }
        theFont.iX+=theFont.iWidth;
        }
    }
