// HEADER - GAME.H
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, August 2012
//
// Common game stuff.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#if !defined(__GAME_H__)
#define __GAME_H__

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <u235se.h>
#include <joypad.h>
#include "objects.h"
#include "BlitBitmap.h"

#if !defined(FALSE)
#define FALSE (0)
#define TRUE (!(FALSE))
#endif

// Font.
#define FONT_USED_IS_SMALL  0
#define FONT_USED_IS_LARGE  1
#define LARGE_FONT_WIDTH_IN_PIXELS 16
#define LARGE_FONT_HEIGHT_IN_PIXELS 16
#define LARGE_FONT_SIZE_IN_BYTES (LARGE_FONT_WIDTH_IN_PIXELS*LARGE_FONT_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define SMALL_FONT_WIDTH_IN_PIXELS 8
#define SMALL_FONT_HEIGHT_IN_PIXELS 8

// Font character indexes.
#define FONT_LETTER_INDEX        0
#define FONT_DIGIT_INDEX         26
#define FONT_PUNCTUATION_INDEX   (FONT_LETTER_INDEX+FONT_DIGIT_INDEX+10)
#define FONT_FULL_STOP_INDEX     (FONT_PUNCTUATION_INDEX+0)
#define FONT_COMMA_INDEX         (FONT_PUNCTUATION_INDEX+1)
#define FONT_SEMI_COLON_INDEX    (FONT_PUNCTUATION_INDEX+2)
#define FONT_COLON_INDEX         (FONT_PUNCTUATION_INDEX+3)
#define FONT_PLING_INDEX         (FONT_PUNCTUATION_INDEX+4)
#define FONT_QUESTION_MARK_INDEX (FONT_PUNCTUATION_INDEX+5)
#define FONT_QUOTE_INDEX         (FONT_PUNCTUATION_INDEX+6)
#define FONT_MINUS_INDEX         (FONT_PUNCTUATION_INDEX+7)
#define FONT_LEFT_BRACKET_INDEX  (FONT_PUNCTUATION_INDEX+8)
#define FONT_RIGHT_BRACKET_INDEX (FONT_PUNCTUATION_INDEX+9)
#define FONT_AMPERSAND_INDEX     (FONT_PUNCTUATION_INDEX+10)

extern unsigned char gameTextIsCentred;
extern void DrawFont(const char* aMsg,short int aX, short int aY);
extern void SetFontSize(unsigned short int aLargeFont);

// Game graphics look.
#define LOOK_IS_RETRO   0
#define LOOK_IS_MODERN  1
extern unsigned char gameDisplayGraphics;

// Game level pack.
#define LEVEL_PACK_IS_NORMAL        0
#define LEVEL_PACK_IS_BIT_TWIDDLERS 1
extern unsigned char gameDisplayLevelPack;

// Volume levels
#define DEFAULT_MUSIC_VOLUME    64
#define DEFAULT_FX_VOLUME       64
#define VOLUME_MULTIPLIER       4
extern unsigned char gameMusicVolume;
extern unsigned char gameFxVolume;

#define DEFAULT_GRAPHICS_LOOK LOOK_IS_MODERN
//#define DEFAULT_GRAPHICS_LOOK LOOK_IS_RETRO

// Control the level pack.
#define DEFAULT_LEVEL_PACK LEVEL_PACK_IS_NORMAL
//#define DEFAULT_LEVEL_PACK LEVEL_PACK_IS_BIT_TWIDDLERS

// Tiles.
#define TILE_WIDTH_IN_PIXELS    16
#define TILE_HEIGHT_IN_PIXELS   8
#define TILE_INDEX_WALL         (0*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_INDEX_LADDER       (1*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_INDEX_EGG          (2*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_INDEX_GRAIN        (3*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_INDEX_BLANK        (4*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_INDEX_CHAIN        (5*TILE_WIDTH_IN_PIXELS*TILE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define TILE_BITMAP_SIZE_IN_BYTES 1536

extern const unsigned char KClassicLookTilesBitmap[TILE_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookTilesBitmap[TILE_BITMAP_SIZE_IN_BYTES];

// Cage.
#define CAGE_WIDTH_IN_PIXELS    48
#define CAGE_HEIGHT_IN_PIXELS   48
#define CAGE_START_X            0
#define CAGE_START_Y            0xdc
#define CAGE_INDEX_CLOSED       (0*CAGE_WIDTH_IN_PIXELS*CAGE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define CAGE_INDEX_OPEN         (1*CAGE_WIDTH_IN_PIXELS*CAGE_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define CAGE_BITMAP_SIZE_IN_BYTES   9216

extern const unsigned char KClassicLookCageBitmap[CAGE_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookCageBitmap[CAGE_BITMAP_SIZE_IN_BYTES];
extern unsigned char gameCageBitmap[CAGE_BITMAP_SIZE_IN_BYTES];
extern void DrawCage(void);

// Digits.
#define DIGIT_WIDTH_IN_PIXELS   10
#define DIGIT_HEIGHT_IN_PIXELS  9
#define DIGIT_SIZE_IN_BYTES     (((DIGIT_WIDTH_IN_PIXELS*DIGIT_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)+7)&0xFFF8)
#define DIGITS_BITMAP_SIZE_IN_BYTES 3680

extern const unsigned char KClassicLookDigitsBitmap[ DIGITS_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookDigitsBitmap[ DIGITS_BITMAP_SIZE_IN_BYTES];
extern unsigned char gameDigitsBitmap[ DIGITS_BITMAP_SIZE_IN_BYTES];

// Tile start
#define TILE_START_Y 49

// HUD.
#define HUD_WIDTH_IN_PIXELS 320
#define HUD_HEIGHT_IN_PIXELS 18
#define HUD_BITMAP_SIZE_IN_BYTES 11520
#define HUD_START_X 0
#define HUD_START_Y 44

#define SCORE_NUMBER_DIGIT_Y HUD_START_Y
#define SCORE_NUMBER_DIGIT1_RETRO_X 116
#define SCORE_NUMBER_DIGIT1_MODERN_X (SCORE_NUMBER_DIGIT1_RETRO_X-4)
#define SCORE_NUMBER_DIGIT1_X theHudScoreXPos

#define MAX_LIVES_TO_DISPLAY 8
#define LIFE_GAP_IN_PIXELS  8
#define LIFE_START_X        114
#define LIFE_START_Y        (HUD_START_Y+DIGIT_HEIGHT_IN_PIXELS+2)

#define LEVEL_NUMBER_DIGIT_Y (HUD_START_Y+DIGIT_HEIGHT_IN_PIXELS)
#define LEVEL_NUMBER_DIGIT1_RETRO_X 194
#define LEVEL_NUMBER_DIGIT1_MODERN_X (LEVEL_NUMBER_DIGIT1_RETRO_X-19)
#define LEVEL_NUMBER_DIGIT1_X theHudLevelXPos
#define LEVEL_NUMBER_DIGIT2_X (theHudLevelXPos+(1*DIGIT_WIDTH_IN_PIXELS))
#define LEVEL_NUMBER_DIGIT3_X (theHudLevelXPos+(2*DIGIT_WIDTH_IN_PIXELS))

#define BONUS_NUMBER_DIGIT_Y (HUD_START_Y+DIGIT_HEIGHT_IN_PIXELS)
#define BONUS_NUMBER_DIGIT1_RETRO_X 237
#define BONUS_NUMBER_DIGIT1_MODERN_X (BONUS_NUMBER_DIGIT1_RETRO_X-12)
#define BONUS_NUMBER_DIGIT1_X theHudBonusXPos
#define BONUS_NUMBER_DIGIT2_X (theHudBonusXPos+(1*DIGIT_WIDTH_IN_PIXELS))
#define BONUS_NUMBER_DIGIT3_X (theHudBonusXPos+(2*DIGIT_WIDTH_IN_PIXELS))
#define BONUS_NUMBER_DIGIT4_X (theHudBonusXPos+(3*DIGIT_WIDTH_IN_PIXELS))

#define TIMER_NUMBER_DIGIT_Y (HUD_START_Y+DIGIT_HEIGHT_IN_PIXELS)
#define TIMER_NUMBER_DIGIT1_RETRO_X 288
#define TIMER_NUMBER_DIGIT1_MODERN_X (TIMER_NUMBER_DIGIT1_RETRO_X-12)
#define TIMER_NUMBER_DIGIT1_X theHudTimerXPos
#define TIMER_NUMBER_DIGIT2_X (theHudTimerXPos+(1*DIGIT_WIDTH_IN_PIXELS))
#define TIMER_NUMBER_DIGIT3_X (theHudTimerXPos+(2*DIGIT_WIDTH_IN_PIXELS))
                                     
extern unsigned char KClassicLookHudBitmap[HUD_BITMAP_SIZE_IN_BYTES];
extern unsigned char KModernLookHudBitmap[HUD_BITMAP_SIZE_IN_BYTES];

// Game over.
#define GAME_OVER_WIDTH_IN_PIXELS 320
#define GAME_OVER_HEIGHT_IN_PIXELS 200

extern const unsigned char KGameOverBitmapData[];

// Get ready.
#define GET_READY_DELAT_IN_DECONDS 4
#define GET_READY_WIDTH_IN_PIXELS 320
#define GET_READY_HEIGHT_IN_PIXELS 200

extern const unsigned char KGetReadyBitmapData[];

// High scores
#define HIGH_SCORES_WIDTH_IN_PIXELS 320
#define HIGH_SCORES HEIGHT_IN_PIXELS 200

extern const unsigned char KHighScoresBitmapData[];

// Credits
#define CREDITS_WIDTH_IN_PIXELS 320
#define CREDITS HEIGHT_IN_PIXELS 200

extern const unsigned char KCreditsBitmapData[];

// Player sprite dimensions.
#define PLAYER_WALKING_WIDTH_IN_PIXELS 16
#define PLAYER_WALKING_HEIGHT_IN_PIXELS 16
#define PLAYER_CLIMBING_WIDTH_IN_PIXELS 16
#define PLAYER_CLIMBING_HEIGHT_IN_PIXELS 18
#define PLAYER_BITMAP_SIZE_IN_BYTES 4800

// Player sprite indexes.
#define PLAYER_INDEX_RIGHT_F1   (0*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_RIGHT_F2   (1*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_RIGHT_F3   (2*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_LEFT_F1    (3*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_LEFT_F2    (4*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_LEFT_F3    (5*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_CLIMB_F1   (6*PLAYER_WALKING_WIDTH_IN_PIXELS*PLAYER_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_CLIMB_F2   PLAYER_INDEX_CLIMB_F1+(1*PLAYER_CLIMBING_WIDTH_IN_PIXELS*PLAYER_CLIMBING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PLAYER_INDEX_CLIMB_F3   PLAYER_INDEX_CLIMB_F1+(2*PLAYER_CLIMBING_WIDTH_IN_PIXELS*PLAYER_CLIMBING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)

extern const unsigned char KClassicLookPlayerBitmap[PLAYER_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookPlayerBitmap[PLAYER_BITMAP_SIZE_IN_BYTES];
extern unsigned char gamePlayerBitmap[PLAYER_BITMAP_SIZE_IN_BYTES];

// Hen sprite dimensions.
#define HEN_WALKING_WIDTH_IN_PIXELS 16
#define HEN_WALKING_HEIGHT_IN_PIXELS 20
#define HEN_CLIMBING_WIDTH_IN_PIXELS 16
#define HEN_CLIMBING_HEIGHT_IN_PIXELS 20
#define HEN_EATING_WIDTH_IN_PIXELS 32
#define HEN_EATING_HEIGHT_IN_PIXELS 20
#define HEN_BITMAP_SIZE_IN_BYTES 8960

// Hen sprite indexes.
#define HEN_INDEX_RIGHT_F1      (0*HEN_WALKING_WIDTH_IN_PIXELS*HEN_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_RIGHT_F2      (1*HEN_WALKING_WIDTH_IN_PIXELS*HEN_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_LEFT_F1       (2*HEN_WALKING_WIDTH_IN_PIXELS*HEN_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_LEFT_F2       (3*HEN_WALKING_WIDTH_IN_PIXELS*HEN_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_CLIMB_F1      (4*HEN_WALKING_WIDTH_IN_PIXELS*HEN_WALKING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_CLIMB_F2      HEN_INDEX_CLIMB_F1+(1*HEN_CLIMBING_WIDTH_IN_PIXELS*HEN_CLIMBING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_EAT_RIGHT_F1  HEN_INDEX_CLIMB_F1+(2*HEN_CLIMBING_WIDTH_IN_PIXELS*HEN_CLIMBING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_EAT_RIGHT_F2  HEN_INDEX_EAT_RIGHT_F1+(1*HEN_EATING_WIDTH_IN_PIXELS*HEN_EATING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_EAT_LEFT_F1   HEN_INDEX_EAT_RIGHT_F1+(2*HEN_EATING_WIDTH_IN_PIXELS*HEN_EATING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define HEN_INDEX_EAT_LEFT_F2   HEN_INDEX_EAT_RIGHT_F1+(3*HEN_EATING_WIDTH_IN_PIXELS*HEN_EATING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)

extern const unsigned char KClassicLookHenBitmap[HEN_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookHenBitmap[HEN_BITMAP_SIZE_IN_BYTES];
extern unsigned char gameHenBitmap[HEN_BITMAP_SIZE_IN_BYTES];

// Mother hen dimensions.
#define MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS     32
#define MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS    24

// Mother hen sprite indexes.
#define MOTHER_HEN_INDEX_RIGHT_F1   (0*MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS*MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define MOTHER_HEN_INDEX_RIGHT_F2   (1*MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS*MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define MOTHER_HEN_INDEX_LEFT_F1    (2*MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS*MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define MOTHER_HEN_INDEX_LEFT_F2    (3*MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS*MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define MOTHER_HEN_BITMAP_SIZE_IN_BYTES 6144

extern const unsigned char KClassicLookMotherHenBitmap[MOTHER_HEN_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookMotherHenBitmap[MOTHER_HEN_BITMAP_SIZE_IN_BYTES];
extern unsigned char gameMotherHenBitmap[MOTHER_HEN_BITMAP_SIZE_IN_BYTES];

// Lift dimensions.
#define LIFT_WIDTH_IN_PIXELS    32
#define LIFT_HEIGHT_IN_PIXELS   20
#define LIFT_BITMAP_SIZE_IN_BYTES 2560
#define LIFT_MAX_ANIMATION_COUNT 5

extern const unsigned char KClassicLookLiftBitmap[LIFT_BITMAP_SIZE_IN_BYTES];
extern const unsigned char KModernLookLiftBitmap[LIFT_BITMAP_SIZE_IN_BYTES];
extern unsigned char gameLiftBitmap[LIFT_BITMAP_SIZE_IN_BYTES];
extern unsigned short gameLiftAnimationFrame;
extern unsigned short gameLiftAnimationCount;

// Pause.
#define PAUSE_MODERN_WIDTH_IN_PIXELS    80
#define PAUSE_MODERN_HEIGHT_IN_PIXELS   13
#define PAUSE_CLASSIC_WIDTH_IN_PIXELS   80
#define PAUSE_CLASSIC_HEIGHT_IN_PIXELS  8
#define PAUSE_MODERN_X ((320-PAUSE_MODERN_WIDTH_IN_PIXELS)/4)
#define PAUSE_MODERN_Y ((256-PAUSE_MODERN_HEIGHT_IN_PIXELS)/2)
#define PAUSE_CLASSIC_X ((320-PAUSE_CLASSIC_WIDTH_IN_PIXELS)/4)
#define PAUSE_CLASSIC_Y ((256-PAUSE_CLASSIC_HEIGHT_IN_PIXELS)/2)
#define PAUSE_MODERN_BITMAP_DATA_INDEX  0
#define PAUSE_CLASSIC_BITMAP_DATA_INDEX  (PAUSE_MODERN_WIDTH_IN_PIXELS*PAUSE_MODERN_HEIGHT_IN_PIXELS*PIXEL_WIDTH_IN_BYTES)
#define PAUSE_FADE_FRACTION 64  // A quarter.
extern const unsigned char KPauseBitmap[];

extern TBitmapObj gameObjBitmap;
extern TBitmapObj gameObjPlayer;
extern TBitmapObj gameObjHens[5];
extern TBitmapObj gameObjMotherHen;
extern TBitmapObj gameObjCage;
extern TBitmapObj gameObjLift1;
extern TBitmapObj gameObjLift2;
extern TBitmapObj gameObjPause;

extern const unsigned char* KNormalLevels[8];
extern const unsigned char* KBitTwiddlersLevels[8];
extern unsigned char* gameLevels[8];

extern const unsigned char KGameClassicLookTextData[];

extern const unsigned char KTitlePageMusic[];

#define TITLE_PAGE_WIDTH_IN_PIXELS 320
#define TITLE_PAGE_HEIGHT_IN_PIXELS 200
extern const unsigned char KTitlePageData[];

// Backdrops
#define BACKDROP_WIDTH_IN_PIXELS        320
#define BACKDROP_HEIGHT_IN_PIXELS       211
#define BACKFROP_MAX_PALETTE_ENTRIES    16
extern const unsigned char KBackDropLevel1Data[];
extern const unsigned char KBackDropLevel2Data[];
extern const unsigned char KBackDropLevel3Data[];
extern const unsigned char KBackDropLevel4Data[];
extern const unsigned char KBackDropLevel5Data[];
extern const unsigned char KBackDropLevel6Data[];
extern const unsigned char KBackDropLevel7Data[];
extern const unsigned char KBackDropLevel8Data[];

extern void DrawClassicLookLife(unsigned char *aPtr);
extern void DrawModernLookLife(unsigned char *aPtr);
extern void ObjLiftsReset(void);
extern void ObjHensReset(void);
extern void ObjPlayerReset(void);
extern void ObjPauseReset(TBitmapObj* anObjPtr);
extern void LoadLevel(void);
extern void StartLevel(void);

extern TBitmapBlitType gameDigitBmp;

// Global variables.
extern volatile unsigned char gameTicks;        // Changed on VBI.
extern volatile unsigned char gameSeconds;      // Changed on VBI.
extern volatile unsigned char gameTickComplete; // Changed on VBI.
extern unsigned char gameTickFrequency;
extern unsigned char gamePlayerHasSelectedStartLevel;
extern unsigned char gamePlayerStartLevel;
extern unsigned char gameDisplayHighScoreLeague;
extern unsigned char gameInBonusMode;
extern unsigned char gameBackdropBuffer[];
extern unsigned char* gameLevelDecompressedPtr;
extern unsigned short int* gameLevelPalettePtr;

// Global functions.
extern void FlipScreens(void);
extern void KillAllSprites(void);
extern void WaitForVBlank(void);
extern void WaitForTwoSeconds(void);
extern void WaitForPauseKeyUp(void);
extern void WaitForPauseKeyDown(void);
extern void DebounceAnyKey(void);
extern void DrawCredits(void);
extern void DrawHighScores(void);
extern void SetPalette(unsigned short int *aPaletteData);
extern unsigned char HandleNewHighScore(void);
extern void InitHudStatPositions(void);
extern void HandleOptions(TBitmapBlitType* aBackgroundBmp);
extern void InitGpu(void);
extern void AddBitmapToDimList(void* aBitmapPtr,
                    unsigned long int aLength,
                    unsigned short int aFraction);
extern void DimBitmapDispatch(void);
extern void AlphaOptionsBar(const unsigned char* aDataPtr);
extern void DecompressBackdrop(unsigned char* aSrc, unsigned short aType);
extern void SooundEffectAccept(void);
extern void SooundEffectNext(void);
extern void SooundEffectCancel(void);
extern void SaveDataToCart(void);
extern void LoadDataFromCart(void);

#endif  // __GAME_H__
