// MODULE - MAIN.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, August 2012
//
// Atari Jaguar Duckie Egg.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"
#include "chuckie.h"
#include "raster.h"

// #define __SHOW_LEVELS_ONLY__

// Title page states
#define TITLE_STATE_SHOW_TITLE      0
#define TITLE_STATE_SHOW_HSC        1
#define TITLE_STATE_SHOW_LEVEL      2
#define TITLE_STATE_SHOW_CREDITS    3

#define TITLE_PAGE_NEXT_STATE_TIME_IN_SECONDS 10

unsigned char gameStartLevel;

// Sprites.
static const TSpriteFrameType thePlayerLeft[4]=
    {
    {&gamePlayerBitmap[PLAYER_INDEX_LEFT_F1],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_LEFT_F2],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_LEFT_F1],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_LEFT_F3],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS}
    };

static const TSpriteFrameType thePlayerRight[4]=
    {
    {&gamePlayerBitmap[PLAYER_INDEX_RIGHT_F1],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_RIGHT_F2],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_RIGHT_F1],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_RIGHT_F3],PLAYER_WALKING_WIDTH_IN_PIXELS,PLAYER_WALKING_HEIGHT_IN_PIXELS}
    };

static const TSpriteFrameType thePlayerClimb[4]=
    {
    {&gamePlayerBitmap[PLAYER_INDEX_CLIMB_F1],PLAYER_CLIMBING_WIDTH_IN_PIXELS,PLAYER_CLIMBING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_CLIMB_F2],PLAYER_CLIMBING_WIDTH_IN_PIXELS,PLAYER_CLIMBING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_CLIMB_F1],PLAYER_CLIMBING_WIDTH_IN_PIXELS,PLAYER_CLIMBING_HEIGHT_IN_PIXELS},
    {&gamePlayerBitmap[PLAYER_INDEX_CLIMB_F3],PLAYER_CLIMBING_WIDTH_IN_PIXELS,PLAYER_CLIMBING_HEIGHT_IN_PIXELS}
    };

static const TSpriteFrameType theHenActions[10]=
    {
    {&gameHenBitmap[HEN_INDEX_RIGHT_F1],HEN_WALKING_WIDTH_IN_PIXELS,HEN_WALKING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_RIGHT_F2],HEN_WALKING_WIDTH_IN_PIXELS,HEN_WALKING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_LEFT_F1],HEN_WALKING_WIDTH_IN_PIXELS,HEN_WALKING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_LEFT_F2],HEN_WALKING_WIDTH_IN_PIXELS,HEN_WALKING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_CLIMB_F1],HEN_CLIMBING_WIDTH_IN_PIXELS,HEN_CLIMBING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_CLIMB_F2],HEN_CLIMBING_WIDTH_IN_PIXELS,HEN_CLIMBING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_EAT_RIGHT_F1],HEN_EATING_WIDTH_IN_PIXELS,HEN_EATING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_EAT_RIGHT_F2],HEN_EATING_WIDTH_IN_PIXELS,HEN_EATING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_EAT_LEFT_F1],HEN_EATING_WIDTH_IN_PIXELS,HEN_EATING_HEIGHT_IN_PIXELS},
    {&gameHenBitmap[HEN_INDEX_EAT_LEFT_F2],HEN_EATING_WIDTH_IN_PIXELS,HEN_EATING_HEIGHT_IN_PIXELS}
    };

static const TSpriteFrameType theLift=
    {
    &gameLiftBitmap[0],LIFT_WIDTH_IN_PIXELS,LIFT_HEIGHT_IN_PIXELS
    };

static const TSpriteFrameType theMotherHenRight[2]=
    {
    {&gameMotherHenBitmap[MOTHER_HEN_INDEX_RIGHT_F1],MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS,MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS},
    {&gameMotherHenBitmap[MOTHER_HEN_INDEX_RIGHT_F2],MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS,MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS}
    };

static const TSpriteFrameType theMotherHenLeft[2]=
    {
    {&gameMotherHenBitmap[MOTHER_HEN_INDEX_LEFT_F1],MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS,MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS},
    {&gameMotherHenBitmap[MOTHER_HEN_INDEX_LEFT_F2],MOTHER_HEN_FLAPPING_WIDTH_IN_PIXELS,MOTHER_HEN_FLAPPING_HEIGHT_IN_PIXELS}
    };

//  Lifts.
unsigned short gameLiftAnimationFrame;
unsigned short gameLiftAnimationCount;
const unsigned  char* const KLiftAnimationFrameData[2]=
    {
    &gameLiftBitmap[0],
    &gameLiftBitmap[LIFT_BITMAP_SIZE_IN_BYTES/2]
    };

unsigned char gameDisplayGraphics=DEFAULT_GRAPHICS_LOOK;
unsigned char gameDisplayLevelPack=DEFAULT_LEVEL_PACK;
unsigned char gamePlayerHasSelectedStartLevel=FALSE;
unsigned char gamePlayerStartLevel=1;
unsigned char gameFrameRateIsSlow=FALSE;
unsigned char gameDisplayHighScoreLeague=FALSE;
unsigned char gameInBonusMode=FALSE;

void KillAllSprites(void)
//
// Kill all the sprite objects.
//
    {

    ObjLiftsReset();
    ObjPlayerReset();
    ObjHensReset();
    }

static void InitSpriteData(void)
//
// Initialise the sprite data.
//
    {

    // Modern or clasic look graphics?
    if (gameDisplayGraphics==LOOK_IS_RETRO)
        {
        // Init retro mode.
        BlitMemCpy64(&gamePlayerBitmap[0],&KClassicLookPlayerBitmap[0],PLAYER_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameLiftBitmap[0],&KClassicLookLiftBitmap[0],LIFT_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameHenBitmap[0],&KClassicLookHenBitmap[0],HEN_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameMotherHenBitmap[0],&KClassicLookMotherHenBitmap[0],MOTHER_HEN_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameCageBitmap[0],&KClassicLookCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameDigitsBitmap[0],&KClassicLookDigitsBitmap[0],DIGITS_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameCageBitmap[0],&KClassicLookCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES);
        }
    else
        {
        // Init modern mode.
        BlitMemCpy64(&gamePlayerBitmap[0],&KModernLookPlayerBitmap[0],PLAYER_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameLiftBitmap[0],&KModernLookLiftBitmap[0],LIFT_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameHenBitmap[0],&KModernLookHenBitmap[0],HEN_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameMotherHenBitmap[0],&KModernLookMotherHenBitmap[0],MOTHER_HEN_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameCageBitmap[0],&KModernLookCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameDigitsBitmap[0],&KModernLookDigitsBitmap[0],DIGITS_BITMAP_SIZE_IN_BYTES);
        BlitMemCpy64(&gameCageBitmap[0],&KModernLookCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES);
        }

    InitHudStatPositions();
    }

void DrawAttractModeLevelDisplay(void)
//
// Display a random level in attract mode.
//
    {

    current_level=rand()&7;

    InitSpriteData();
    memcpy(&gameLevels[0],(gameDisplayLevelPack==LEVEL_PACK_IS_NORMAL)?&KNormalLevels[0]:&KBitTwiddlersLevels[0],8*sizeof(unsigned char*));
    LoadLevel();
    StartLevel();
    }

void DrawTitle(void)
//
// Put up the title page.
//
    {
    TBitmapBlitType bmp=
        {
        (unsigned char*)&KTitlePageData[0],
        NULL,
        WID320,
        LFU_REPLACE,
        TITLE_PAGE_WIDTH_IN_PIXELS,TITLE_PAGE_HEIGHT_IN_PIXELS,
        (SCREEN_WIDTH_IN_PIXELS-TITLE_PAGE_WIDTH_IN_PIXELS)/2,
        SCREEN_CROP_HEIGHT_IN_PIXELS+((SCREEN_HEIGHT_VISIBLE_IN_PIXELS-TITLE_PAGE_HEIGHT_IN_PIXELS)/2)
        };
    unsigned char state=-1;
    unsigned char newState=TRUE;

    // Check if we should display the high score league.
    if (gameDisplayHighScoreLeague)
        {
        gameDisplayHighScoreLeague=FALSE;   // One shot.
        state=TITLE_STATE_SHOW_TITLE;       // From this state.
        }

    // Default start level.
    gameStartLevel=0;
    gameSeconds=0;

    // Wait for the player to start.
    for (;;)
        {
        // Check for a change of state.
        if (newState)
            {
            newState=FALSE;
            gameSeconds=0;

            // Move to next state.
            if (++state>TITLE_STATE_SHOW_CREDITS)
                state=TITLE_STATE_SHOW_TITLE;

            // Clear the screen.
            BlitClearScreen();

            // Nuke the sprites
            KillAllSprites();

            // Stop displaying the level backdrop.
            gameObjBitmap.iPhrase2.iTrans=0;    // Disable transparency.

            // Handle new state.
            bmp.iDest=gameBackScreen;
            switch (state)
                {
            case TITLE_STATE_SHOW_TITLE:
                bmp.iData=(unsigned char*)&KTitlePageData[0];
                BlitBitmap(&bmp);
            break;
            case TITLE_STATE_SHOW_HSC:
                bmp.iData=(unsigned char*)&KHighScoresBitmapData[0];
                BlitBitmap(&bmp);
                DrawHighScores();
            break;
            case TITLE_STATE_SHOW_LEVEL:
                DrawAttractModeLevelDisplay();
            break;
            case TITLE_STATE_SHOW_CREDITS:
                bmp.iData=(unsigned char*)&KCreditsBitmapData[0];
                BlitBitmap(&bmp);
                DrawCredits();
            break;
                }

             // Flip it to the foreground.
            if (state!=TITLE_STATE_SHOW_LEVEL)
                FlipScreens();
            WaitForVBlank();
            }

        WaitForVBlank();
        unsigned long int what=JoyPad();

        // Next state timer?
        if (gameSeconds==TITLE_PAGE_NEXT_STATE_TIME_IN_SECONDS)
            newState=TRUE;
        
        // Slow mode?
        gameFrameRateIsSlow=(what&JOYPAD_KEY0)?TRUE:FALSE;

        if (what&JOYPAD_FIRE)
            break;

        // Starting on alternate levels?
        if (what&JOYPAD_KEY1)
            {
            gameStartLevel=1*8; // 9
            gamePlayerHasSelectedStartLevel=FALSE;
            break;
            }
        else if (what&JOYPAD_KEY2)
            {
            gameStartLevel=2*8; // 17
            gamePlayerHasSelectedStartLevel=FALSE;
            break;
            }
        else if (what&JOYPAD_KEY3)
            {
            gameStartLevel=3*8; // 25
            gamePlayerHasSelectedStartLevel=FALSE;
            break;
            }
        else if (what&JOYPAD_KEY4)
            {
            gameStartLevel=4*8; // 33
            gamePlayerHasSelectedStartLevel=FALSE;
            break;
            }
        else if (what&JOYPAD_OPTION)
            {
            bmp.iData=(unsigned char*)&KCreditsBitmapData[0];
            HandleOptions(&bmp);
            newState=TRUE;
            }
        }

    // Initialise the sprite data.
    InitSpriteData();

    // Which level pack?
    memcpy(&gameLevels[0],(gameDisplayLevelPack==LEVEL_PACK_IS_NORMAL)?&KNormalLevels[0]:&KBitTwiddlersLevels[0],8*sizeof(unsigned char*));

    // Cheating?
    if (gamePlayerHasSelectedStartLevel)
        gameStartLevel=gamePlayerStartLevel-1;

    gameLevelDecompressedPtr=NULL;
    }

void DrawGetReady(void)
//
// Put up the "GET READY!" page.
//
    {
    TBitmapBlitType bmp=
        {
        (unsigned char*)&KGetReadyBitmapData[0],
        NULL,
        WID320,
        LFU_REPLACE,
        GET_READY_WIDTH_IN_PIXELS,GET_READY_HEIGHT_IN_PIXELS,
        (SCREEN_WIDTH_IN_PIXELS-GET_READY_WIDTH_IN_PIXELS)/2,
        SCREEN_CROP_HEIGHT_IN_PIXELS+((SCREEN_HEIGHT_VISIBLE_IN_PIXELS-GET_READY_HEIGHT_IN_PIXELS)/2)
        };

    WaitForVBlank();

    // Clear the screen.
    BlitClearScreen();

    // Blit the background image.
    bmp.iDest=gameBackScreen;
    BlitBitmap(&bmp);
    gameObjBitmap.iPhrase2.iTrans=0;    // Disable transparency.

    WaitForVBlank();

    // Nuke the sprites.
    KillAllSprites();

    // Flip it to the foreground.
    FlipScreens();
    WaitForVBlank();

    // Wait for 30 seconds or fire.
    gameTicks=0;
    gameSeconds=0;
    unsigned char quitOnFire=FALSE;
    for (;;)
        {
        WaitForVBlank();
        if (gameSeconds==GET_READY_DELAT_IN_DECONDS)
            break;
        if (JoyPad()&JOYPAD_FIRE)
            {
            quitOnFire=TRUE;
            break;
            }
        }

    // Wait for fire button(s) to come up.
    if (quitOnFire)
        for (;;)
            if ((JoyPad()&JOYPAD_FIRE)==0)
                break;
    }

void DrawGameOver(void)
//
// Put up the "GAME OVER" page.
//
    {
    TBitmapBlitType bmp=
        {
        (unsigned char*)&KGameOverBitmapData[0],
        NULL,
        WID320,
        LFU_REPLACE,
        GAME_OVER_WIDTH_IN_PIXELS,GAME_OVER_HEIGHT_IN_PIXELS,
        (SCREEN_WIDTH_IN_PIXELS-GAME_OVER_WIDTH_IN_PIXELS)/2,
        SCREEN_CROP_HEIGHT_IN_PIXELS+((SCREEN_HEIGHT_VISIBLE_IN_PIXELS-GAME_OVER_HEIGHT_IN_PIXELS)/2)
        };

    WaitForVBlank();

    // Clear the screen.
    BlitClearScreen();

    // Blit the background image.
    bmp.iDest=gameBackScreen;
    BlitBitmap(&bmp);
    gameObjBitmap.iPhrase2.iTrans=0;    // Disable transparency.

    // Nuke the sprites.
    WaitForVBlank();
    KillAllSprites();

    // Flip it to the foreground.
    FlipScreens();
    WaitForVBlank();

    // Let player take finger of jump/directions whatever.
    WaitForTwoSeconds();

    // Did the player get a new high score?
    if (!HandleNewHighScore())
        {
        // Wait for 30 seconds or fire.
        gameSeconds=0;
        for (;;)
            {
            WaitForVBlank();
            if (gameSeconds==30)
                break;
            if (JoyPad()&JOYPAD_FIRE)
                break;
            }
        }

    // Wait for pad directions/fire to be released.
    for (;;)
        {
        WaitForVBlank();
        if (JoyPad()==0)
            break;
        }
    }

void DoPause(void)
//
// Handle pause/unpause
//
    {
    static const TSpriteFrameType pauseModernSpr=
        {
        &KPauseBitmap[PAUSE_MODERN_BITMAP_DATA_INDEX],PAUSE_MODERN_WIDTH_IN_PIXELS,PAUSE_MODERN_HEIGHT_IN_PIXELS
        };
    static const TSpriteFrameType pauseClassicSpr=
        {
        &KPauseBitmap[PAUSE_CLASSIC_BITMAP_DATA_INDEX],PAUSE_CLASSIC_WIDTH_IN_PIXELS,PAUSE_CLASSIC_HEIGHT_IN_PIXELS
        };
    unsigned short int paletteCopy[BACKFROP_MAX_PALETTE_ENTRIES];

    // Wait for PAUSE to be released.
    WaitForPauseKeyUp();

    // Take a copy of the backdrop palette.
    for (short int c=0; c<BACKFROP_MAX_PALETTE_ENTRIES; c++)
        paletteCopy[c]=gameLevelPalettePtr[c];

    // Fade the background down.
    BlitFrontToBack();
    AddBitmapToDimList(gameBackScreen+(SCREEN_CROP_HEIGHT_IN_PIXELS*SCREEN_WIDTH_IN_BYTES),SCREEN_HEIGHT_VISIBLE_IN_PIXELS*SCREEN_WIDTH_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&paletteCopy[0],BACKFROP_MAX_PALETTE_ENTRIES*2,PAUSE_FADE_FRACTION);

    // Fade the sprites down.
    AddBitmapToDimList(&gamePlayerBitmap[0],PLAYER_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameLiftBitmap[0],LIFT_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameHenBitmap[0],HEN_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameMotherHenBitmap[0],MOTHER_HEN_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameDigitsBitmap[0],DIGITS_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);
    AddBitmapToDimList(&gameCageBitmap[0],CAGE_BITMAP_SIZE_IN_BYTES,PAUSE_FADE_FRACTION);

    // Make sure everything faded during VBLANK time.
    WaitForVBlank();
    DimBitmapDispatch();
    SetPalette(&paletteCopy[0]);

    // Activate the pause object.
    TBitmapObj* pPauseObj=&gameObjHens[num_ducks];
    if (gameDisplayGraphics==LOOK_IS_RETRO)
        MakeBitmapObj(pPauseObj,&pauseClassicSpr,PAUSE_CLASSIC_X,PAUSE_CLASSIC_Y);
    else
        MakeBitmapObj(pPauseObj,&pauseModernSpr,PAUSE_MODERN_X,PAUSE_MODERN_Y);

    // Flip to the front.
    FlipScreens();
    WaitForVBlank();

    // Music and FX off.
    SoundVolumeMusic(0);
    SoundVolumeFx(0);

    // Wait for PAUSE to be pressed.
    WaitForPauseKeyDown();

    // Wait for PAUSE to be released (prevents us going right back into pause).
    WaitForPauseKeyUp();

    // Remove PAUSE object.
    ObjPauseReset(pPauseObj);

    // Recolour the sprites.
    InitSpriteData();

    // Restore the front again.
    FlipScreens();
    WaitForVBlank();

    // Reset the backdrop palette.
    SetPalette(gameLevelPalettePtr);

    // Drawing on front buffer again.
    gameDigitBmp.iDest=gameFrontScreen;

    // Sound back to normal.
    SoundVolumeMusic(gameMusicVolume);
    SoundVolumeFx(gameFxVolume);
    }

void PollKeys(void)
//
//
//
    {
    const unsigned long int what=JoyPad();

    buttons=0;

    if (what&JOYPAD_FIREB)
	  buttons|=BUTTON_JUMP;

    if (what&JOYPAD_LEFT)
        buttons|=BUTTON_LEFT;
    else if (what&JOYPAD_RIGHT)
        buttons|=BUTTON_RIGHT;

    if (what&JOYPAD_UP)
        buttons|=BUTTON_UP;
    else if (what&JOYPAD_DOWN)
        buttons|=BUTTON_DOWN;

    if (what&JOYPAD_RESET)      // Need to reset?
        buttons|=BUTTON_QUIT;
    else if (what&JOYPAD_PAUSE) // Need to pause?
        DoPause();
    }

void InitPlayer(void)
//
// Initialise the player sprite.
//
    {

    MakeBitmapObj(&gameObjPlayer,&thePlayerLeft[0],player_x,player_y);
    }

void InitDucks(void)
//
// Initialise the hens.
//
    {
    TBitmapObj* pHenObj=&gameObjHens[0];

    // Stop objects in all the hens just in case.
    ObjHensReset();

    if (num_ducks)
        for (short int i=0; i<num_ducks; i++)
            {
            short int x=duck[i].x;
            short int sprite=DuckSprite(i);
            if (sprite>=8)  // Eating and facing left?
                x-=8;

            MakeBitmapObj(pHenObj,&theHenActions[sprite],x,duck[i].y);
            pHenObj++;
            }

    // Mother hen
    MakeBitmapObj(&gameObjMotherHen,&theMotherHenRight[0],big_duck_x,big_duck_y);

    // Draw the cage.
    DrawCage();
    }

void InitLifts(void)
//
// Initialise the lifts.
//
    {

    // Branch over (just in case).
    ObjLiftsReset();

    if (have_lift)
        {
        MakeBitmapObj(&gameObjLift1,&theLift,lift_x,lift_y[0]);
        MakeBitmapObj(&gameObjLift2,&theLift,lift_x,lift_y[1]);
        }
    }

void RenderFrame(void)
//
// Move/animate sprites.
//
    {
    const TSpriteFrameType* pPlayerSprite;
    unsigned short frame;

    // Draw the player.
    if (player_face==0)
        {
        pPlayerSprite=&thePlayerClimb[0];
        frame=(player_y>>1)&3;
        }
    else
        {
        if (player_face<0)
            pPlayerSprite=&thePlayerLeft[0];
        else
            pPlayerSprite=&thePlayerRight[0];
        frame=(player_x>>1)&3;
        }

    if (player_mode!=PLAYER_CLIMB)
        {
        if (move_x==0)
	        frame=0;
        }
    else
        {
        if (move_y==0)
	        frame=0;
        }

    // Adjust player position, sprite animation frame and sprite height.
    gameObjPlayer.iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-player_y)<<1))&0xFFFE;
    gameObjPlayer.iPhrase1.iHeight=pPlayerSprite[frame].iHeightInPixels;
    gameObjPlayer.iPhrase1.iData=((unsigned long int)pPlayerSprite[frame].iData)>>3;  // In phrases.
    gameObjPlayer.iPhrase2.iXPos=gameObjListOffsetX+(player_x<<1);

    // Move/animate the hens.
    TBitmapObj* pHenObj=&gameObjHens[0];
    for (short int i=0; i<num_ducks; i++)
        {
        short int x=duck[i].x;
        short int sprite=DuckSprite(i);
        if (sprite>=8)  // Eating and facing left?
            x-=8;

        pHenObj->iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-duck[i].y)<<1))&0xFFFE;
        pHenObj->iPhrase1.iHeight=theHenActions[sprite].iHeightInPixels;
        pHenObj->iPhrase1.iData=((unsigned long int)theHenActions[sprite].iData)>>3;  // In phrases.
        pHenObj->iPhrase2.iDataWidth=theHenActions[sprite].iWidthInPixels>>2; // In phrases (@16BPP).
        pHenObj->iPhrase2.iImageWidth=theHenActions[sprite].iWidthInPixels>>2;// In phrases (@16BPP).
        pHenObj->iPhrase2.iXPos=gameObjListOffsetX+(x<<1);

        pHenObj++;
        }

    // Move/animate the mother hen.
    const TSpriteFrameType* pMotherHen;
    if (big_duck_dir)
        pMotherHen=&theMotherHenLeft[big_duck_frame];
    else
        pMotherHen=&theMotherHenRight[big_duck_frame];
    gameObjMotherHen.iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-big_duck_y)<<1))&0xFFFE;
    gameObjMotherHen.iPhrase1.iData=((unsigned long int)pMotherHen->iData)>>3;  // In phrases.
    gameObjMotherHen.iPhrase2.iXPos=gameObjListOffsetX+(big_duck_x<<1);

    // Animate the lifts (if any).
    if (have_lift)
        {
        ;
        const unsigned  long int liftData=((unsigned long int)KLiftAnimationFrameData[gameLiftAnimationFrame])>>3;
        gameObjLift1.iPhrase1.iData=liftData;
        gameObjLift2.iPhrase1.iData=liftData;
        gameObjLift1.iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-lift_y[0])<<1))&0xFFFE;
        gameObjLift2.iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-lift_y[1])<<1))&0xFFFE;
        if (gameLiftAnimationCount--==0)
            {
            gameLiftAnimationCount=LIFT_MAX_ANIMATION_COUNT;
            ++gameLiftAnimationFrame;
            gameLiftAnimationFrame&=1;
            }
        }

    SoundFxQueueDispatch();
    WaitForVBlank();
    if (gameFrameRateIsSlow&&!gameInBonusMode)
        WaitForVBlank();
    }

int main(int argc, const char *argv[])
//
// The main ingredient.
//
    {
    TU235SoundEngineModType snd=
        {
        &KTitlePageMusic[0],
        U235SE_16KHZ,
        U235SE_16KHZ_PERIOD,
        NULL
        };

    // Fire up TOM.
    InitGpu();

#if defined(__SHOW_LEVELS_ONLY__)
    InitSpriteData();
    memcpy(&gameLevels[0],(gameDisplayLevelPack==LEVEL_PACK_IS_NORMAL)?&KNormalLevels[0]:&KBitTwiddlersLevels[0],8*sizeof(unsigned char*));
    LoadLevel();
    unsigned long int what;
    for (;;)
        {
        for (;;)
            {
            WaitForVBlank();
            what=JoyPad();
            if (what&(JOYPAD_KEY1|JOYPAD_KEY2|JOYPAD_KEY3|JOYPAD_KEY4|JOYPAD_KEY5|JOYPAD_KEY6|JOYPAD_KEY7|JOYPAD_KEY8))
                break;
            }
        if (what&JOYPAD_KEY1)
            current_level=0;
        if (what&JOYPAD_KEY2)
            current_level=1;
        if (what&JOYPAD_KEY3)
            current_level=2;
        if (what&JOYPAD_KEY4)
            current_level=3;
        if (what&JOYPAD_KEY5)
            current_level=4;
        if (what&JOYPAD_KEY6)
            current_level=5;
        if (what&JOYPAD_KEY7)
            current_level=6;
        if (what&JOYPAD_KEY8)
            current_level=7;

        BlitClearScreen();
        LoadLevel();
        }
#endif

    // If the player is holding down E2PROM erase keys on the jag pad
    // we'll just go with the defaults and overwite the flash.
    if ((JoyPad()&JOYPAD_ERASE_EPROM)==0)
        LoadDataFromCart();
    else
        SaveDataToCart();

    // Start some tunage!
    if (SoundModInit(&snd)==0)
        {
        SoundModPlayMono();
        SoundVolumeFx(gameFxVolume);
        SoundVolumeMusic(gameMusicVolume);
        }

    run_game();

    return(0);
    }
