// MODULE - SCREEN.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, August 2012
//
// Background/hud.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "chuckie.h"
#include "game.h"

TBitmapBlitType gameDigitBmp=
    {
    NULL,
    NULL,
    WID10,
    LFU_REPLACE,
    DIGIT_WIDTH_IN_PIXELS,DIGIT_HEIGHT_IN_PIXELS,
    0,0
    };

unsigned short int* gameLevelPalettePtr=NULL;
unsigned char* gameLevelDecompressedPtr=NULL;

// Cage sprites.
static const TSpriteFrameType theCageClosed=
    {
    &gameCageBitmap[CAGE_INDEX_CLOSED],CAGE_WIDTH_IN_PIXELS,CAGE_HEIGHT_IN_PIXELS
    };

static const TSpriteFrameType theCageOpen=
    {
    &gameCageBitmap[CAGE_INDEX_OPEN],CAGE_WIDTH_IN_PIXELS,CAGE_HEIGHT_IN_PIXELS
    };

// HUD positions
static unsigned short int theHudScoreXPos;
static unsigned short int theHudLevelXPos;
static unsigned short int theHudBonusXPos;
static unsigned short int theHudTimerXPos;

void RemoveTile(unsigned short aX, unsigned short aY)
//
// Remove a tile from the foreground.
//
    {
    TBitmapBlitType bmp=
        {
        (unsigned char*)&KClassicLookTilesBitmap[TILE_INDEX_BLANK],
        NULL,
        WID16,
        LFU_REPLACE,
        16,8,
        0,0
        };

    bmp.iDest=gameFrontScreen;
    bmp.iX=aX<<4;
    bmp.iY=TILE_START_Y+(((25-aY)<<3)|7);

    BlitBitmap(&bmp);
    }

static void DrawDigit(unsigned short int aX, unsigned short int aY, unsigned char aDigit)
//
// Draw a digit.
//
    {

    gameDigitBmp.iData=&gameDigitsBitmap[aDigit*DIGIT_SIZE_IN_BYTES];
    gameDigitBmp.iX=aX;
    gameDigitBmp.iY=aY;
    BlitBitmap(&gameDigitBmp);
    }

static void DrawScoreDigit(unsigned short int aX, unsigned short int aY, unsigned char aDigit)
//
// Draw a score digit.
//
    {

    gameDigitBmp.iData=&gameDigitsBitmap[(aDigit*DIGIT_SIZE_IN_BYTES)+(10*DIGIT_SIZE_IN_BYTES)];
    gameDigitBmp.iX=aX;
    gameDigitBmp.iY=aY;
    BlitBitmap(&gameDigitBmp);
    }

void DrawScore(int n, int oldval, int newval)
//
// Draw a digit of the score.
//
    {

    if (n>=3)
        DrawScoreDigit((SCORE_NUMBER_DIGIT1_X-(2*DIGIT_WIDTH_IN_PIXELS))+(n*DIGIT_WIDTH_IN_PIXELS),SCORE_NUMBER_DIGIT_Y,newval);
    }

void DrawTimer(int n)
//
// Draw a timer digit.
//
    {

    DrawDigit(TIMER_NUMBER_DIGIT1_X+(n*DIGIT_WIDTH_IN_PIXELS),TIMER_NUMBER_DIGIT_Y,timer_ticks[n]);
    }

void DrawBonus(int n)
//
// Draw a bonus digit.
//
    {

    DrawDigit(BONUS_NUMBER_DIGIT1_X+(n*DIGIT_WIDTH_IN_PIXELS),BONUS_NUMBER_DIGIT_Y,bonus[n]);
    }

void DrawLife(int n)
//
//
//
    {

    // Too many lives on screen already?
    if (n>MAX_LIVES_TO_DISPLAY)
        return;

    unsigned char *pDest=gameBackScreen+
                        (LIFE_START_Y*SCREEN_WIDTH_IN_BYTES)+
                        (LIFE_START_X-
                        LIFE_GAP_IN_PIXELS+
                        (n*LIFE_GAP_IN_PIXELS))*PIXEL_WIDTH_IN_BYTES;

    if (gameDisplayGraphics==LOOK_IS_RETRO)
        DrawClassicLookLife(pDest);
    else
        DrawModernLookLife(pDest);
    }

void InitHudStatPositions(void)
//
// Initialise the hud positions.
//
    {

    if (gameDisplayGraphics==LOOK_IS_RETRO)
        {
        theHudScoreXPos=SCORE_NUMBER_DIGIT1_RETRO_X;
        theHudLevelXPos=LEVEL_NUMBER_DIGIT1_RETRO_X;
        theHudBonusXPos=BONUS_NUMBER_DIGIT1_RETRO_X;
        theHudTimerXPos=TIMER_NUMBER_DIGIT1_RETRO_X;
        }
    else
        {
        theHudScoreXPos=SCORE_NUMBER_DIGIT1_MODERN_X;
        theHudLevelXPos=LEVEL_NUMBER_DIGIT1_MODERN_X;
        theHudBonusXPos=BONUS_NUMBER_DIGIT1_MODERN_X;
        theHudTimerXPos=TIMER_NUMBER_DIGIT1_MODERN_X;
        }
    }

void DrawBackDrop(void)
//
//
//
    {
    static const char* const KBackDropBiatmaps[10]=
        {
        &KBackDropLevel1Data[0],
        &KBackDropLevel2Data[0],
        &KBackDropLevel3Data[0],
        &KBackDropLevel4Data[0],
        &KBackDropLevel5Data[0],
        &KBackDropLevel6Data[0],
        &KBackDropLevel7Data[0],
        &KBackDropLevel8Data[0],
        };

    // Only backdrops on the modern look, normal level pack.
    if ((gameDisplayGraphics!=LOOK_IS_MODERN)||(gameDisplayLevelPack!=LEVEL_PACK_IS_NORMAL))
        return;

    // Decompression needed
    unsigned short *pBackdrop=(unsigned short*)KBackDropBiatmaps[current_level&7];
    unsigned short type=*pBackdrop++;

    // Set palette.
    gameLevelPalettePtr=pBackdrop;
    SetPalette(gameLevelPalettePtr);

    // Draw the backdrop (only if this is a fresh level).
    unsigned char *pImg=gameBackdropBuffer;
    unsigned char* pSrc=(unsigned char*)&pBackdrop[16];
    if (pSrc!=gameLevelDecompressedPtr)  // Already decompacted?
        {
        gameLevelDecompressedPtr=pSrc;
        DecompressBackdrop(pSrc,type);
        }
    }

void DrawCage(void)
//
// Draw the cage.
//
    {

    MakeBitmapObj(&gameObjCage,have_big_duck?&theCageClosed:&theCageOpen,CAGE_START_X,CAGE_START_Y);
    }

void RenderBackground(void)
//
// Render the background.
//
    {
    TBitmapBlitType bmp=
        {
        NULL,
        NULL,
        WID16,
        LFU_REPLACE,
        16,8,
        0,0
        };

    // Draw the backdrop.
    DrawBackDrop();

    bmp.iDest=gameBackScreen;
    for (unsigned char x=0; x<20; x++)
        {
        bmp.iX=x<<4;
        for (unsigned char y=0; y<25; y++)
            {
    	    unsigned char type=levelmap[(y*20)+x];
            unsigned short int bmpDataIndex=0;

	        if (type&TILE_LADDER)
        		bmpDataIndex=TILE_INDEX_LADDER;
            else if (type&TILE_WALL)
        		bmpDataIndex=TILE_INDEX_WALL;
            else if (type&TILE_EGG)
        		bmpDataIndex=TILE_INDEX_EGG;
            else if (type&TILE_GRAIN)
        		bmpDataIndex=TILE_INDEX_GRAIN;
            else
        		continue;

            bmp.iData=(gameDisplayGraphics==LOOK_IS_RETRO)?(unsigned char*)&KClassicLookTilesBitmap[bmpDataIndex]:(unsigned char*)&KModernLookTilesBitmap[bmpDataIndex];
            bmp.iY=TILE_START_Y+(((25-y)<<3)|7);
            BlitBitmap(&bmp);
        	}
        }

    // Draw the hud.
    bmp.iData=(gameDisplayGraphics==LOOK_IS_RETRO)?&KClassicLookHudBitmap[0]:&KModernLookHudBitmap[0];
    bmp.iFlags2=WID320;
    bmp.iWidth=HUD_WIDTH_IN_PIXELS;
    bmp.iHeight=HUD_HEIGHT_IN_PIXELS;
    bmp.iX=HUD_START_X;
    bmp.iY=HUD_START_Y;
    BlitBitmap(&bmp);

    // Populate the hud.
    gameDigitBmp.iDest=gameBackScreen;

    // Score.
    unsigned short i=0;
    for ( ; i<6; i++) 
        DrawScoreDigit(SCORE_NUMBER_DIGIT1_X+(i*10),SCORE_NUMBER_DIGIT_Y,all_player_data[0].score[i+2]);

    // Lives
    i=all_player_data[0].lives;
    if (i>=2)
        {
        if (--i>MAX_LIVES_TO_DISPLAY)
            i=MAX_LIVES_TO_DISPLAY;
        unsigned char *pDest=gameBackScreen+(LIFE_START_Y*SCREEN_WIDTH_IN_BYTES)+(LIFE_START_X*PIXEL_WIDTH_IN_BYTES);
        if (gameDisplayGraphics==LOOK_IS_RETRO)
            while (i--)
                {
                DrawClassicLookLife(pDest);
                pDest+=LIFE_GAP_IN_PIXELS*PIXEL_WIDTH_IN_BYTES;
                }
        else
            while (i--)
                {
                DrawModernLookLife(pDest);
                pDest+=LIFE_GAP_IN_PIXELS*PIXEL_WIDTH_IN_BYTES;
                }

        }

    // Level number.
    short int tmp=current_level+1;
    DrawDigit(LEVEL_NUMBER_DIGIT3_X,LEVEL_NUMBER_DIGIT_Y,tmp%10);
    tmp/=10;
    DrawDigit(LEVEL_NUMBER_DIGIT2_X,LEVEL_NUMBER_DIGIT_Y,tmp%10);
    if (tmp>10)
        DrawDigit(LEVEL_NUMBER_DIGIT1_X,LEVEL_NUMBER_DIGIT_Y,tmp/10);

    // Bonus
    DrawDigit(BONUS_NUMBER_DIGIT1_X,BONUS_NUMBER_DIGIT_Y,bonus[0]);
    DrawDigit(BONUS_NUMBER_DIGIT2_X,BONUS_NUMBER_DIGIT_Y,bonus[1]);
    DrawDigit(BONUS_NUMBER_DIGIT3_X,BONUS_NUMBER_DIGIT_Y,bonus[2]);
    DrawDigit(BONUS_NUMBER_DIGIT4_X,BONUS_NUMBER_DIGIT_Y,0);

    // Timer.
    DrawDigit(TIMER_NUMBER_DIGIT1_X,TIMER_NUMBER_DIGIT_Y,timer_ticks[0]);
    DrawDigit(TIMER_NUMBER_DIGIT2_X,TIMER_NUMBER_DIGIT_Y,0);
    DrawDigit(TIMER_NUMBER_DIGIT3_X,TIMER_NUMBER_DIGIT_Y,0);

    // Draw the cage.
    DrawCage();

    // If its the modern look and the normal level pack enable transparency
    // so the backdrop shows through.
    if ((gameDisplayGraphics==LOOK_IS_MODERN)&&(gameDisplayLevelPack==LEVEL_PACK_IS_NORMAL))
        gameObjBitmap.iPhrase2.iTrans=1;

    // Flip it to foreground!
    FlipScreens();
    WaitForVBlank();

    // Digits on the front screen from now on.
    gameDigitBmp.iDest=gameFrontScreen;

    // Reset lift animation.
    gameLiftAnimationFrame=0;
    gameLiftAnimationCount=LIFT_MAX_ANIMATION_COUNT;
    }

