// MODULE - SOUND.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, August 2012
//
// Sound handling.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"

#define PLAYER_MOVE_VOICE_MASK (SFX_VOICE5_MASK|SFX_VOICE7_MASK)
#define EGG_VOICE_MASK (SFX_VOICE6_MASK|SFX_VOICE8_MASK)
#define GRAIN_VOICE_MASK (SFX_VOICE6_MASK|SFX_VOICE8_MASK)
#define DEATH_TUNE_VOICE_MASK (PLAYER_MOVE_VOICE_MASK|EGG_VOICE_MASK|GRAIN_VOICE_MASK)

extern const TU235SoundEngineSampleBankType gameSampleBank[];
extern const TU235SoundEngineSampleBankType gameSoundFxWalk;
extern const TU235SoundEngineSampleBankType gameSoundFxLadder;
extern const TU235SoundEngineSampleBankType gameSoundFxJump;
extern const TU235SoundEngineSampleBankType gameSoundFxFall;
extern const TU235SoundEngineSampleBankType gameSoundFxEgg;
extern const TU235SoundEngineSampleBankType gameSoundFxGrain;
extern const TU235SoundEngineSampleBankType gameSoundFxDeathTune;
extern const TU235SoundEngineSampleBankType gameSoundFxSilence;
extern const TU235SoundEngineSampleBankType gameSoundFxBonus;
extern const TU235SoundEngineSampleBankType gameSoundFxAccept;
extern const TU235SoundEngineSampleBankType gameSoundFxNext;
extern const TU235SoundEngineSampleBankType gameSoundFxCancel;

unsigned char gameMusicVolume=DEFAULT_MUSIC_VOLUME;
unsigned char gameFxVolume=DEFAULT_FX_VOLUME;

static const TU235SoundEngineSoundEffectType sndPlayWalk=
    {
    &gameSampleBank[0],
    &gameSoundFxWalk,
    0,
    PLAYER_MOVE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayLadder=
    {
    &gameSampleBank[0],
    &gameSoundFxLadder,
    0,
    PLAYER_MOVE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayJump=
    {
    &gameSampleBank[0],
    &gameSoundFxJump,
    0,
    PLAYER_MOVE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayFall=
    {
    &gameSampleBank[0],
    &gameSoundFxFall,
    0,
    PLAYER_MOVE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayEgg=
    {
    &gameSampleBank[0],
    &gameSoundFxEgg,
    0,
    EGG_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayGrain=
    {
    &gameSampleBank[0],
    &gameSoundFxGrain,
    0,
    GRAIN_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayDeathTune=
    {
    &gameSampleBank[0],
    &gameSoundFxDeathTune,
    0,
    DEATH_TUNE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlaySilence=
    {
    &gameSampleBank[0],
    &gameSoundFxSilence,
    0,
    PLAYER_MOVE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlaySilenceAll=
    {
    &gameSampleBank[0],
    &gameSoundFxSilence,
    0,
    DEATH_TUNE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayBonus=
    {
    &gameSampleBank[0],
    &gameSoundFxBonus,
    0,
    DEATH_TUNE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayAccept=
    {
    &gameSampleBank[0],
    &gameSoundFxAccept,
    0,
    DEATH_TUNE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayNext=
    {
    &gameSampleBank[0],
    &gameSoundFxNext,
    0,
    DEATH_TUNE_VOICE_MASK
    };

static const TU235SoundEngineSoundEffectType sndPlayCancel=
    {
    &gameSampleBank[0],
    &gameSoundFxCancel,
    0,
    DEATH_TUNE_VOICE_MASK
    };

void SoundEffectWalking(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayWalk);
    }

void SoundEffectLadder(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayLadder);
    }

void SoundEffectFall(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayFall);
    }

void SoundEffectJump(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayJump);
    }

void SoundEffectSilence(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlaySilence);
    }

void SoundEffectSilenceAll(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlaySilenceAll);
    }

void SoundEffectEgg(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayEgg);
    }

void SoundEffectGrain(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayGrain);
    }

void SoundEffectDeathTune(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayDeathTune);
    SoundFxQueueDispatch();

    // Knock off the music while the tune plays.
    SoundVolumeMusic(0);

    // Can't play the tune asynchronously so wait for 2.5 seconds.
    // Wait for 30 seconds or fire.
    gameSeconds=0;
    gameTicks=0;
    for (;;)
        {
        WaitForVBlank();
        if ((gameSeconds==2)&&(gameTicks==(gameTickFrequency>>1)))
            break;
        }

    // Music back on.
    SoundVolumeMusic(gameMusicVolume);
    }

void SoundEffectBonusStart(void)
//
//
//
    {

    gameTicks=0;
    gameSeconds=0;
    SoundFxQueuePlay(&sndPlayBonus);
    }

void SooundEffectBonus(void)
//
//
//
    {

    if (gameTicks==(gameTickFrequency>>1))
        {
        gameTicks=0;
        gameSeconds=0;
        SoundFxQueuePlay(&sndPlayBonus);
        }
    }

void SooundEffectAccept(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayAccept);
    SoundFxQueueDispatch();
    }

void SooundEffectNext(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayNext);
    SoundFxQueueDispatch();
    }

void SooundEffectCancel(void)
//
//
//
    {

    SoundFxQueuePlay(&sndPlayCancel);
    SoundFxQueueDispatch();
    }

