; MODULE - STARTUP.S
;
; Copyright (c) 2012 Mark Ball. All rights reserved.
; Started by Mark Ball, August 2012
;
; Start up sequence.
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;

; ------------------------------------------------------------------
; Includes.
; ------------------------------------------------------------------
	.include "jaguar.inc"
    .include "game.inc"

; ------------------------------------------------------------------
; Imports
; ------------------------------------------------------------------
    .extern	_main
    .extern _InitStandardLib
    .extern _BSS_END

; ------------------------------------------------------------------
; Equates.
; ------------------------------------------------------------------

; ------------------------------------------------------------------
; Code starts here.
; ------------------------------------------------------------------
	.text
    .68000

    .globl RomEntryPoint
RomEntryPoint:
    move.w #$2700, sr               ; No 68K interrupts.

    move.l #0, G_CTRL
    move.l #0, D_CTRL
    move.l #0, G_FLAGS
    move.l #0, D_FLAGS

    move.l #$70007, G_END           ; big-endian mode
    move.l #$70007, D_END
    move.w #$FFFF, VI               ; disable video interrupts

    move.l #INITSTACK, a7   	    ; Setup a stack

; Initialising the DATA section and clearing the BSS section may take
; some time and may also overwrite the stop object written to the OP by
; the BIOS. To circumvent that we'll set up a temporary object list
; containing a STOP object in the base of the RAM.
    jsr InitTempStopObject

; Initialise the standard library.
    jsr _InitStandardLib

; Copy GPU code to the right place.
    move.l #GpuSetOLP, a0
    move.l #G_RAM, a1
    move.w #((GpuCodeEnd-GpuCodeStart)/4)-1, d0
.CopyGpuCodeLoop:
    move.l (a0)+, (a1)+
    dbra d0, .CopyGpuCodeLoop

    jsr InitVideo           ; Setup video regs.
    jsr InitObjectList      ; Initialise Object Display List
    jsr InitVBlankIrq       ; Initialise our VBLANK routine

; Sneaky trick to cause display to popup at first VB
    move.l #$0, _gameObjListCopy+(FirstDisplayableObject-_gameObjList)
    move.l #$C, _gameObjListCopy+(FirstDisplayableObject-_gameObjList)+4

    move.l #G_RAM, G_PC   	    ; Set GPU PC
    move.l #RISCGO, G_CTRL      ; Go!
.WaitForGpuDone:
    move.l G_CTRL, d0
    andi.l #$1, d0
    bne.s .WaitForGpuDone

    move.w #$6C7, VMODE     	; Configure Video

; Jump to main code
    jsr _main

; Should never get here!

; Wait until the cows come home!
.WaitUntilTheCoesComeHome:
    bra.s .WaitUntilTheCoesComeHome

    .long
; ------------------------------------------------------------------
; GPU code follows.
; ------------------------------------------------------------------
    .gpu

; ------------------------------------------------------------------
; GpuSetOLP
; ------------------------------------------------------------------
GpuSetOLP:
    .org G_RAM

; Set OLP to the object list
GpuCodeStart:
    movei #olp2set, r0
    load (r0), r1
    movei #OLP, r0
    store r1, (r0)

; Stop GPU.
    moveq #0, r0
    movei #G_CTRL, r1
    store r0, (r1)

; Two "feet" on the brake pedal
    nop             		
    nop

    .long                   ; Ensure word boundary.
GpuCodeEnd:

; ------------------------------------------------------------------
; Code starts again.
; ------------------------------------------------------------------
    .68000

; ------------------------------------------------------------------
; InitVBlankIrq
; ------------------------------------------------------------------
; Install the VBLANK Irq handler and enable interrupts.
; ------------------------------------------------------------------
InitVBlankIrq:

; Install IRQ handler
    move.l #HandleVBlankIrq, LEVEL0

    move.w a_vde, d0        	; Must be ODD
    ori.w #1, d0
    move.w d0, VI

; Aknowledge *ALL* interrupts and enable VBLANK IRQ.
    move.w #C_JERCLR|C_PITCLR|C_OPCLR|C_GPUCLR|C_VIDCLR|C_VIDENA, INT1
    move.w #0, INT2

; Lower 68k IPL to allow interrupts.
    and.w #$F8FF, sr

    rts
		
; ------------------------------------------------------------------
; InitVideo
; ------------------------------------------------------------------
; ------------------------------------------------------------------
InitVideo:

; PAL or NTSC?
    move.w CONFIG, d0      		 ; Also is joystick register
	andi.w #VIDTYPE, d0    		 ; 0 = PAL, 1 = NTSC
    beq.s .palvals

    move.b #60, _gameTickFrequency

    move.w #NTSC_HMID, d2
    move.w #NTSC_WIDTH, d0
    move.w #NTSC_VMID, d6
    move.w #NTSC_HEIGHT, d4

	bra.s .calc_vals
.palvals:
    move.b #50, _gameTickFrequency

	move.w #PAL_HMID, d2
	move.w #PAL_WIDTH, d0
	move.w #PAL_VMID, d6
	move.w #PAL_HEIGHT, d4

.calc_vals:
    move.w d0, width
    move.w d4, height

    move.w d0, d1
    asr.w #1, d1            ; Width/2

    sub.w d1, d2            ; Mid - Width/2
    add.w #4, d2            ; (Mid - Width/2)+4

    sub.w #1, d1            ; Width/2 - 1
    ori.w #$400, d1         ; (Width/2 - 1)|$400
		
    move.w d1, a_hde
    move.w d1, HDE

    move.w d2, a_hdb
    move.w d2, HDB1
    move.w d2, HDB2

    move.w d6, d5
    sub.w d4, d5

    move.w d5, a_vdb

    add.w d4, d6
    move.w d6, a_vde
    move.w d5, VDB
    move.w #$FFFF, VDE
	
    move.l #0, BORD1        	; Black border
    move.w #0, BG           	; Init line buffer to black
	
    rts

; ------------------------------------------------------------------
; InitTempStopObject
; ------------------------------------------------------------------
; ------------------------------------------------------------------
InitTempStopObject:

; Create a pointer to where the temporary stop object will be.
    clr.l a0

    move.l a0, BORD1        ; Black border.
    move.w a0, BG           ; Init line buffer to black.

; Create a STOP object.
    clr.l (a0)+             ; Bits 32 to 63 zero.
    move.l #STOPOBJ, (a0)   ; STOP object and bits 3 to 31 zero.

; Init Object Processor
    move.l a0, OLP          ; NOTE: No IRQs enabled so safe to do this.
    move.w a0, OBF

; Ack all pending interrupts (if any).
    move.w #C_JERCLR|C_PITCLR|C_OPCLR|C_GPUCLR|C_VIDCLR, INT1
    move.w #0, INT2

; No VI.
    move.w #$7FFF, VI

    rts

; ------------------------------------------------------------------
; InitObjectList
; ------------------------------------------------------------------
; ------------------------------------------------------------------
InitObjectList:
    lea _gameObjList, a0

    move.l #_gameObjListCopy+(StopObj-_gameObjList), d2  ; Address of STOP object
    move.l d2, d3			        ; Copy for low half

    lsr.l #8, d2			        ; Shift high half into place
    lsr.l #3, d2

    swap d3			                ; Place low half correctly
    clr.w d3
    lsl.l #5, d3

; Write first BRANCH object (branch if YPOS > a_vde )
    clr.l d0
    move.l #(BRANCHOBJ|O_BRLT), d1  ; $4000 = VC < YPOS
    or.l d2, d0			            ; Do LINK overlay
    or.l d3, d1
						
    move.w a_vde, d4                ; for YPOS
    lsl.w #3, d4                    ; Make it bits 13-3
    or.w d4, d1

    move.l d0, (a0)+
    move.l d1, (a0)+

; Write second branch object (branch if YPOS < a_vdb)
; Note: LINK address is the same so preserve it
    andi.l #$FF000007, d1       ; Mask off CC and YPOS
    ori.l #O_BRGT, d1      	    ; $8000 = VC > YPOS
    move.w a_vdb, d4            ; for YPOS
    lsl.w #3, d4                ; Make it bits 13-3
    or.w d4, d1

    move.l d0, (a0)+
    move.l d1, (a0)+

; Game backdrop
; Write a standard BITMAP object
    moveq.l #24, d5
    move.l #_gameObjListCopy+(_gameObjBitmap-_gameObjList), a2
    move.l a2, d1
    lsr.l #3, d1
    lsl.l d5, d1
    move.l a2, d0
    lsr.l #3, d0
    lsr.l #8, d0

    ori.l #GAME_BACKDROP_HEIGHT_IN_PIXELS<<14, d1  ; Height of image
    move.w height, d4           	        ; Center bitmap vertically
    sub.w #SCREEN_HEIGHT_VISIBLE_IN_PIXELS,d4
    add.w a_vdb, d4
    move.w d4, _gameObjListOffsetY
    andi.w #$FFFE, d4                       ; Must be even
    lsl.w #3, d4
    or.w d4, d1                             ; Stuff YPOS in low phrase

    move.l #_gameBackdropBuffer, d4
    lsl.l #8, d4
    or.l d4, d0

    move.l d0, (a0)+
    move.l d1, (a0)+

; Second Phrase of Bitmap
    move.l #(GAME_BACKDROP_WIDTH_IN_PHRASES>>4), d0	; Only part of top LONG is IWIDTH
    move.l #GAME_BACKDROP_OBJECT_COLOUR_DEPTH|O_NOGAP, d1  ; Bit Depth, Contiguous data

    move.w width, d4            	        ; Get width in clocks
    sub.w #GAME_BACKDROP_WIDTH_IN_PIXELS*4, d4
    lsr.w #3, d4               	            ; /4 Pixel Divisor
    move.w d4, _gameObjListOffsetX
    or.w d4, d1

    ori.l #(GAME_BACKDROP_WIDTH_IN_PHRASES<<18)|(GAME_BACKDROP_WIDTH_IN_PHRASES<<28), d1	; DWIDTH|IWIDTH

    move.l d0, (a0)+
    move.l d1, (a0)+

; Game double buffer bitmap.
; Write a standard BITMAP object
    moveq.l #24, d5
    move.l #_gameObjListCopy+(MovingObjects-_gameObjList), a2
    move.l a2, d1
    lsr.l #3, d1
    lsl.l d5, d1
    move.l a2, d0
    lsr.l #3, d0
    lsr.l #8, d0

    ori.l #SCREEN_HEIGHT_VISIBLE_IN_PIXELS<<14, d1  ; Height of image
    move.w height, d4           	        ; Center bitmap vertically
    sub.w #SCREEN_HEIGHT_VISIBLE_IN_PIXELS,d4
    add.w a_vdb, d4
    move.w d4, _gameObjListOffsetY
    andi.w #$FFFE, d4                       ; Must be even
    lsl.w #3, d4
    or.w d4, d1                             ; Stuff YPOS in low phrase

    move.l _gameFrontScreen, d4
    add.l #SCREEN_HEIGHT_CROP_IN_PIXELS*SCREEN_WIDTH_IN_BYTES, d4
    lsl.l #8, d4
    or.l d4, d0

    move.l d0, (a0)+
    move.l d1, (a0)+

; Second Phrase of Bitmap
    move.l #(SCREEN_WIDTH_IN_PHRASES>>4), d0	; Only part of top LONG is IWIDTH
    move.l #SCREEN_OBJECT_COLOUR_DEPTH|O_NOGAP, d1  ; Bit Depth, Contiguous data

    move.w width, d4            	        ; Get width in clocks
    sub.w #SCREEN_WIDTH_IN_PIXELS*4, d4
    lsr.w #3, d4               	            ; /4 Pixel Divisor
    move.w d4, _gameObjListOffsetX
    or.w d4, d1

    ori.l #(SCREEN_WIDTH_IN_PHRASES<<18)|(SCREEN_WIDTH_IN_PHRASES<<28), d1	; DWIDTH|IWIDTH

    move.l d0, (a0)+
    move.l d1, (a0)+

; Write STOP objects to unused sprites and also to the end of the object list.
    move.l #STOPOBJ, d0
    move.l #_gameObjPlayer, a0
    move.l d0, _gameObjLift1-MovingObjects+4(a0)    ; Lift 1.
    move.l d0, _gameObjLift2-MovingObjects+4(a0)    ; Lift 2.
    move.l d0, _gameObjPlayer-MovingObjects+4(a0)   ; Player.
    move.l d0, _gameObjMotherHen-MovingObjects+4(a0); Mother hen.
    move.l d0, _gameObjCage-MovingObjects+4(a0)     ; Cage.
    move.l d0, _gameObjHen1-MovingObjects+4(a0)     ; Hen 1.
    move.l d0, _gameObjHen2-MovingObjects+4(a0)     ; Hen 2.
    move.l d0, _gameObjHen3-MovingObjects+4(a0)     ; Hen 3.
    move.l d0, _gameObjHen4-MovingObjects+4(a0)     ; Hen 4.
    move.l d0, _gameObjHen5-MovingObjects+4(a0)     ; Hen 5.
    move.l d0, _gameObjPause-MovingObjects+4(a0)    ; PAUSE.
    move.l d0, StopObj-MovingObjects+4(a0)          ; Stop object.

; Copy the game object list to the OP's list.
    lea _gameObjList, a0
    lea _gameObjListCopy, a1
    move.w #((_gameObjListEnd-_gameObjList)/4)-1, d0
.CopyObjectLoop:
    move.l (a0)+, (a1)+
    dbra d0, .CopyObjectLoop 

; Ensure lift branches are taken.
    move.w #$1, OBF

; Word swapped object list address.
    move.l #_gameObjListCopy, d0
    swap d0
    move.l d0, olp2set

    rts

; ------------------------------------------------------------------
; HandleVBlankIrq
; ------------------------------------------------------------------
; Handle the VBLANK IRQ.
; ------------------------------------------------------------------
HandleVBlankIrq:
    movem.l d0/a0-a1, -(sp)

; Copy the game object list to the OP's list.
    lea _gameObjList, a0
    lea _gameObjListCopy, a1
    move.w #((_gameObjListEnd-_gameObjList)/4)-1, d0
.IrqCopyObjectLoop:
    move.l (a0)+, (a1)+
    dbra d0, .IrqCopyObjectLoop 

; One more tick has gone by.
    move.b _gameTicks, d0
    addq.b #1, d0
    cmp.b _gameTickFrequency, d0
    bne.s .SkipSecondsAdjustment
; Notch up another second.
    clr.b d0
    addq.b #1, _gameSeconds
.SkipSecondsAdjustment:
    move.b d0, _gameTicks

; Tick completed its work.
    move.b #1, _gameTickComplete

; Enable and acknowledge the interrupt.
    move.w #C_VIDCLR|C_VIDENA, INT1

    movem.l (sp)+, d0/a0-a1

    move.w #$0, INT2
    rte

; ------------------------------------------------------------------
; _FlipScreens
; ------------------------------------------------------------------
; Flip the front/back screens.
; ------------------------------------------------------------------
    .globl _FlipScreens
_FlipScreens:
    move.l _gameFrontScreen, d1
    move.l _gameBackScreen, d0
    move.l d1, _gameBackScreen      ; Front to back.
    move.l d0, _gameFrontScreen     ; Back to front.
    add.l #SCREEN_HEIGHT_CROP_IN_PIXELS*SCREEN_WIDTH_IN_BYTES, d0
    lsl.l #8, d0
    move.l _gameObjBitmap, d1       ; Upper half of phrase.
    and.l #$000007FF, d1            ; Mask out old DATA address
    or.l d1, d0                     ; Or in the new DATA address
	move.l d0, _gameObjBitmap       ; Update the upper phrase.
    rts

; ------------------------------------------------------------------
; RAM - DATA
; ------------------------------------------------------------------
    .data
    .globl _gameFrontScreen
_gameFrontScreen:   dc.l theScreen1Data
    .globl _gameBackScreen
_gameBackScreen:    dc.l theScreen2Data

; ------------------------------------------------------------------
; RAM - BSS
; ------------------------------------------------------------------
    .bss

; Object list the game manipulates.
    .phrase
    .globl _gameObjList
_gameObjList:
BranchObj1:         ds.b BRANCH_OBJECT_SIZE_IN_BYTES
BranchObj2:         ds.b BRANCH_OBJECT_SIZE_IN_BYTES
FirstDisplayableObject:
    .globl _gameObjBackdrop
_gameObjBackdrop:   ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjBitmap
_gameObjBitmap:     ds.b BITMAP_OBJECT_SIZE_IN_BYTES
MovingObjects:
    .globl _gameObjHighScoreNameEntry
_gameObjHighScoreNameEntry:
    .globl _gameObjLift1, _gameObjLift2
_gameObjLift1:      ds.b BITMAP_OBJECT_SIZE_IN_BYTES
_gameObjLift2:      ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjPlayer
_gameObjPlayer:     ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjMotherHen
_gameObjMotherHen:  ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjCage
_gameObjCage:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjHens, _gameObjHen1, _gameObjHen2, _gameObjHen3, _gameObjHen4, _gameObjHen5
_gameObjHens:
_gameObjHen1:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
_gameObjHen2:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
_gameObjHen3:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
_gameObjHen4:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
_gameObjHen5:       ds.b BITMAP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjPause
_gameObjPause:      ds.b BITMAP_OBJECT_SIZE_IN_BYTES
StopObj:            ds.b STOP_OBJECT_SIZE_IN_BYTES
    .globl _gameObjListEnd
_gameObjListEnd:

; Object list for the Object Processor.
    .dphrase
    .globl _gameObjListCopy, _gameObjListCopyEnd
_gameObjListCopy:        ds.b _gameObjListEnd-_gameObjList
_gameObjListCopyEnd:

olp2set:    ds.l 1  ;  GPU Code Parameter

; Video params.
a_hdb:  ds.w 1
a_hde:  ds.w 1
a_vdb:  ds.w 1
a_vde:  ds.w 1
width:  ds.w 1
height: ds.w 1

; Object offsets.
    .globl _gameObjListOffsetX
_gameObjListOffsetX:    ds.w 1
    .globl _gameObjListOffsetY
_gameObjListOffsetY:    ds.w 1

; Time tracking.
    .globl _gameTickFrequency
_gameTickFrequency:     ds.b 1  ; Tick frequency.
    .globl _gameTicks
_gameTicks:             ds.b 1  ; Tick count.
    .globl _gameSeconds
_gameSeconds:           ds.b 1  ; Seconds count.
    .globl _gameTickComplete
_gameTickComplete:      ds.b 1  ; Tick completed.

; Video buffers.
    .phrase
    .globl _gameBackdropBuffer
_gameBackdropBuffer:    ds.b GAME_BACKDROP_WIDTH_IN_BYTES*GAME_BACKDROP_HEIGHT_IN_PIXELS
theScreen1Data:         ds.b SCREEN_WIDTH_IN_BYTES*SCREEN_HEIGHT_IN_PIXELS
theScreen2Data:         ds.b SCREEN_WIDTH_IN_BYTES*SCREEN_HEIGHT_IN_PIXELS

; ------------------------------------------------------------------
; END
; ------------------------------------------------------------------
		.end

