; MODULE - TILESA.S
;
; Copyright (c) 2012 Mark Ball. All rights reserved.
; Started by Mark Ball, August 2012
;
; Tile data.
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;

; ------------------------------------------------------------------
; Equates.
; ------------------------------------------------------------------
DIGITS_BITMAP_SIZE_IN_BYTES equ 3680
RETRO_LIFE_COLOUR           equ ($1F<<11)|($3F<<0)  ; Yellow!

; ------------------------------------------------------------------
; Code starts here.
; ------------------------------------------------------------------
    .text
    .68000

; ------------------------------------------------------------------
; _DrawClassicLookLife
; ------------------------------------------------------------------
; Draw the classic look's player life indicator.
; ------------------------------------------------------------------
; extern void DrawClassicLookLife(unsigned char* aBitmapPtr);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _DrawClassicLookLife

    .cargs #4, aBitmapPtr.l

_DrawClassicLookLife:
    move.l aBitmapPtr(sp), a0
    move.l #RETRO_LIFE_COLOUR|(RETRO_LIFE_COLOUR<<16), d0
    move.l d0, 4(a0)
    move.l d0, 640(a0)
    move.l d0, 644(a0)
    move.l d0, 648(a0)
    rts

; ------------------------------------------------------------------
; _DrawModernLookLife
; ------------------------------------------------------------------
; Draw the modern look's player life indicator.
; ------------------------------------------------------------------
; extern void DrawModernLookLife(unsigned char* aBitmapPtr);
; ------------------------------------------------------------------
;
; In :-
;   Nothing.
;
; Out :-
;   Nothing.
;
; Trashes :-
;   ABI compliant.
;
; ------------------------------------------------------------------
    .globl _DrawModernLookLife

_DrawModernLookLife:
    move.l aBitmapPtr(sp), a0
    move.l #$90169016, d0
; R1
    move.l d0, 4(a0)
; R2
    move.l d0, 640(a0)
    move.l #$D05DD05D, 640+4(a0)
    move.l d0, 640+8(a0)
; R3
    move.l #$500B500B, 1280+2(a0)
    move.l #$E3E1E3E1, 1280+6(a0)
; R4
    move.w #$500B, 1920+2(a0)
    move.l #$E3E1E3E1, 1920+4(a0)
    rts

; ------------------------------------------------------------------
; Tile bitmaps - retro look.
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KClassicLookTilesBitmap
_KClassicLookTilesBitmap:
    .incbin "..\\data\\gfx\\retro\\tiles.gfx"

; ------------------------------------------------------------------
; Tile bitmaps - modern look.
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KModernLookTilesBitmap
_KModernLookTilesBitmap:
    .incbin "..\\data\\gfx\\modern\\tiles.gfx"

; ------------------------------------------------------------------
; HUD bitmap - retro look.
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KClassicLookHudBitmap
_KClassicLookHudBitmap:
    .incbin "..\\data\\gfx\\retro\\hud.gfx"

; ------------------------------------------------------------------
; HUD bitmap - modern look.
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KModernLookHudBitmap
_KModernLookHudBitmap:
    .incbin "..\\data\\gfx\\modern\\hud.gfx"

; ------------------------------------------------------------------
; HUD digit bitmaps - retro look
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KClassicLookDigitsBitmap
_KClassicLookDigitsBitmap:
    .incbin "..\\data\\gfx\\retro\\digits.gfx"

; ------------------------------------------------------------------
; HUD digit bitmaps - modern look
; ------------------------------------------------------------------
    .text
    .phrase
    .globl _KModernLookDigitsBitmap
_KModernLookDigitsBitmap:
    .incbin "..\\data\\gfx\\modern\\digits.gfx"

; ------------------------------------------------------------------
; RAM - BSS
; ------------------------------------------------------------------
    .bss
    .phrase
    .globl _gameDigitsBitmap
_gameDigitsBitmap:
    ds.b DIGITS_BITMAP_SIZE_IN_BYTES

; ------------------------------------------------------------------
; END
; ------------------------------------------------------------------
    .end

