// MODULE - UTILS.C
//
// Copyright (c) 2012 Mark Ball. All rights reserved.
// Started by Mark Ball, September 2012
//
// Utility functions.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "game.h"

// Debounce ticks.
#define DEBOUNCE_TICKS 5

#pragma dontwarn 208 // Suspicious loop
void die(const char *msg, ...)
//
// Red screen of death. Should never happen!!
//
    {

    *(unsigned short int *)0xf00400=0x1F<<11;
    for (;;)
        ;
    }
#pragma popwarn // Suspicious loop

void abort(void)
//
//
//
    {

    die(NULL);
    }

void WaitForVBlank(void)
//
// Wait for VBLANK to happen.
//
    {

    gameTickComplete=FALSE;

    for (;;)
        if (gameTickComplete)
            break;
    }

void MakeBitmapObj(TBitmapObj* aBmp,const TSpriteFrameType* aSprite,unsigned short int aX, unsigned short int aY)
//
// Create a bitmap object.
//
// Notes
// =====
// Not exactly fast with all the 64 bit "helper bloat" but its only called
// once per sprite at the start of the level.
//
    {
    TBitmapObj obj=*aBmp;

    // Quickest way to clear everything out.
    unsigned long int *pD=(unsigned long int*)&obj;
    pD[0]=0x0;
    pD[1]=0x0;
    pD[2]=0x0;
    pD[3]=0x0;

    // Phrase 1.
    obj.iPhrase1.iYPos=(gameObjListOffsetY+((SPRITE_REFERENCE_Y-aY)<<1))&0xFFFE;
    obj.iPhrase1.iHeight=aSprite->iHeightInPixels;

    // LINK is relative to the destination object list.
    obj.iPhrase1.iLink=((unsigned long int)gameObjListCopy+
                        ((unsigned long int)aBmp-
                        (unsigned long int)gameObjList)+
                        sizeof(TBitmapObj))>>3;                 // In phrases.

    obj.iPhrase1.iData=((unsigned long int)aSprite->iData)>>3;  // In phrases.

    // Phrase 2.
    obj.iPhrase2.iXPos=gameObjListOffsetX+(aX<<1);      // X position x 2.
    obj.iPhrase2.iDepth=4;                              // 16BPP.
    obj.iPhrase2.iPitch=1;                              // Contiguous.
    obj.iPhrase2.iDataWidth=aSprite->iWidthInPixels>>2; // In phrases (@16BPP).
    obj.iPhrase2.iImageWidth=aSprite->iWidthInPixels>>2;// In phrases (@16BPP).
    obj.iPhrase2.iTrans=1;                              // Transparent.

    *aBmp=obj;
    }

void SetPalette(unsigned short int *aPaletteData)
//
// Set the palette for the backdrop.
//
    {
    unsigned short int *pClut=(unsigned short int*)CLUT;

    for (short int c=0; c<BACKFROP_MAX_PALETTE_ENTRIES; c++)
        *pClut++=*aPaletteData++;
    }

void WaitForTwoSeconds(void)
//
// Wait for 2 seconds.
//
    {

    gameSeconds=0;
    gameTicks=0;

    for (;;)
        {
        WaitForVBlank();
        if (gameSeconds==2)
            break;
        }
    }

void WaitForPauseKeyUp(void)
//
// Wait for the psue key to go up.
//
    {

    for (;;)
        {
        WaitForVBlank();
        if ((JoyPad()&JOYPAD_PAUSE)==0)
            break;
        }
    }

void WaitForPauseKeyDown(void)
//
// Wait for the pause key to go down.
//
    {

    for (;;)
        {
        WaitForVBlank();
        if ((JoyPad()&JOYPAD_PAUSE)!=0)
            break;
        }
    }

void DebounceAnyKey(void)
//
// Debounce any key.
//
    {

    gameSeconds=0;
    gameTicks=0;

    for (;;)
        {
        WaitForVBlank();
        if (gameTicks==DEBOUNCE_TICKS)
            break;
        }
    }
